/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*--------------------------------------------------------------------
                         -- C N R S --
         Laboratoire d'automatique et d'analyse des systemes
            Groupe Robotique et Intelligenxce Artificielle
                   7 Avenue du colonel Roche
                     31 077 Toulouse Cedex

  Fichier              : tpStruct.h
  Fonction             : Structures definition for 3D Motion Trajectory Planning
  Date de creation     : Juin 2005      
  Date de modification : Juin 2005
  Nb de lignes         : 

  Auteur               : Ignacio HERRERA AGUILAR

----------------------------------------------------------------------*/


#ifndef tp_struct_h
#define tp_struct_h

#include "../xarmConst.h"

#define MAX_ELEMENTS_LIMIT 10000
#define MAX_POINTS_IN_TRAJECTORY 300
#define MAX_COLS_MATRIX_LIMIT 7

typedef enum TP_STATUS {
  TP_OK    = 0,
  TP_ERROR = 1
} TP_STATUS;

typedef enum TP_SELECT {
  TP_ON  = 0,
  TP_OFF = 1
} TP_SELECT;  /* ON = 0, OFF = 1 */

typedef struct TP_POS {
  double x;
  double y;
  double z;
} TP_POS;

typedef struct TP_ORIGIN_QUAT {
  double x;
  double y;
  double z;
  double ex;
  double ey;
  double ez;
  double angle;
} TP_ORIGIN_QUAT;

typedef struct TP_ABSOLUT_POSITION {
  double x;
  double y;
  double z;
  double theta;
} TP_ABSOLUT_POSITION;

typedef struct TP_QUAT {
  double n;
  double i;
  double j;
  double k;
} TP_QUAT;

typedef struct TP_POSE {
  TP_POS  p;
  TP_QUAT q;
} TP_POSE;

typedef struct TP_VEL {
  double vx;
  double vy;
  double vz;
  double wx;
  double wy;
  double wz;
} TP_VEL;

typedef struct TP_LIMITS {
  double maxJerk;
  double maxAcc;
  double maxVel;
} TP_LIMITS;

typedef struct TP_POSELIMITS {
  TP_LIMITS linear;
  TP_LIMITS angular;
} TP_POSELIMITS;

typedef struct TP_COMPLEX {
  double re;            /* Real Part */
  double im;            /* Imaginary Part */
  double mod;           /* Module */
  double teta;          /* Angle  */
} TP_COMPLEX;

typedef struct TP_TIMES {
  double Tjpa;
  double Taca;
  double Tjna;
  double Tvc;
  double Tjnb;
  double Tacb;
  double Tjpb;
} TP_TIMES;

typedef struct TP_TIMES_GLOBAL {
  double T1;
  double T2;
  double T3;
  double T4;
  double T5;
  double T6;
  double T7;
} TP_TIMES_GLOBAL;

typedef struct TP_AXIS_TIMES {
  TP_TIMES PX;
  TP_TIMES PY;
  TP_TIMES PZ;
  TP_TIMES QN;
  TP_TIMES QI;
  TP_TIMES QJ;
  TP_TIMES QK;
} TP_AXIS_TIMES;

typedef struct TP_COND {
  double a;             /* Acceleration */
  double v;             /* Velocity     */ 
  double x;             /* Distance     */
} TP_COND;

typedef struct TP_JERKS {
  int sel;              /* Sel = 1   J1=J2=J3=J4    */
                        /* Sel = 4   J1, J2, J3, J4 */
  double J1;
  double J2;
  double J3;
  double J4;
  int withOutUseAlignment;
} TP_JERKS;

typedef struct TP_VROW {
  int cols;
  int withOutUseAlignment;
  double el[1][MAX_ELEMENTS_LIMIT];
} TP_VROW;

typedef struct TP_GLOBALVECTORS {
  TP_VROW PX_Acc, PX_Vel, PX_Pos;  
  TP_VROW PY_Acc, PY_Vel, PY_Pos;  
  TP_VROW PZ_Acc, PZ_Vel, PZ_Pos;  
  TP_VROW QN_Acc, QN_Vel, QN_Pos;  
  TP_VROW QI_Acc, QI_Vel, QI_Pos;  
  TP_VROW QJ_Acc, QJ_Vel, QJ_Pos; 
  TP_VROW QK_Acc, QK_Vel, QK_Pos;   
} TP_VECTORS;

typedef struct TP_MAT {
  int rows;
  int cols;
  double el[MAX_ELEMENTS_LIMIT][MAX_COLS_MATRIX_LIMIT];
} TP_MAT;

typedef struct TP_TRAJ {
  int rows;
  int cols;
  double el[MAX_POINTS_IN_TRAJECTORY][MAX_COLS_MATRIX_LIMIT];
} TP_TRAJ;

typedef struct TP_PARTICULAR_VELOCITY {
  double Vsmm;
  double Vsmp;
  double Vs0m;
  double Vs0p;
  double Vfmm;
  double Vfmp;
  double Vf0m;
  double Vf0p;  
  double Vlim;
} TP_PARTICULAR_VELOCITY;

typedef struct TP_VECTOR {
  double Acc;
  double Vel;
  double Pos;
} TP_VECTOR;

typedef struct TP_LOCALVECTORS {
  TP_VECTOR X;
  TP_VECTOR Y;
  TP_VECTOR Z;
  TP_VECTOR QN;
  TP_VECTOR QI;
  TP_VECTOR QJ;
  TP_VECTOR QK;
} TP_LOCALVECTORS;

typedef struct TP_TICK {
  int X;
  int Y;
  int Z;
  int QN;
  int QI;
  int QJ;
  int QK;
} TP_TICK;

typedef struct TP_MOTION {
  TP_TIMES TNE; /* use this only for point to point motion */
  TP_TIMES Acc[XARM_NB_DIM];
  TP_TIMES Vel[XARM_NB_DIM];
  TP_TIMES Pos[XARM_NB_DIM];
  TP_JERKS jerk[XARM_NB_DIM];
  TP_COND  IC[XARM_NB_DIM];
  TP_COND  FC[XARM_NB_DIM];
  int      Dir[XARM_NB_DIM];
  int      MotionDuration[XARM_NB_DIM];
} TP_MOTION;

typedef struct TP_TRANSITION_MOTION {
  TP_TIMES Times[XARM_NB_DIM]; /* en seconde */
  TP_TIMES TimesM[XARM_NB_DIM];
  TP_TIMES Acc_a[XARM_NB_DIM];
  TP_TIMES Vel_a[XARM_NB_DIM];
  TP_TIMES Pos_a[XARM_NB_DIM];
  TP_JERKS jerk[XARM_NB_DIM];
  TP_TIMES Acc_b[XARM_NB_DIM];
  TP_TIMES Vel_b[XARM_NB_DIM];
  TP_TIMES Pos_b[XARM_NB_DIM];
  TP_TIMES Acc[XARM_NB_DIM];
  TP_TIMES Vel[XARM_NB_DIM];
  TP_TIMES Pos[XARM_NB_DIM];
  int      Dir[XARM_NB_DIM];
  TP_COND  IC[XARM_NB_DIM];
  TP_COND  FC[XARM_NB_DIM];
  int      Dir_a[XARM_NB_DIM];
  int      Dir_b[XARM_NB_DIM];
  int      MotionDuration[XARM_NB_DIM];
  int      optimalTime[XARM_NB_DIM]; /* en ticks */
  int      timeToStop;
} TP_TRANSITION_MOTION;

#endif
