/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** xarmCntrlTaskCodels.c
 **
 ** Codels used by the control task xarmCntrlTask
 **
 ** Author: 
 ** Date: 
 **
 **/



# include <portLib.h>
#include "stdio.h"
#include <stdlib.h>
#include "server/xarmHeader.h"
#include "Proto_gb.h"
#include "xarmFunctions.h"
#include "cFunctions.h"
#include "xarmConst.h"
#include <math.h>
#include "server/qarmPosterLib.h"
#include "matrix.h"
#include "queue.h"

static POSTER_ID        posterPlanTraj = NULL;         /* Poster ID for Planned Trajectory */
static XARM_TRAJECTORY  planTraj;                      /* Planned Trajectory */
extern Gb_th* xarmR6Rt;
 
/*------------------------------------------------------------------------
 * xarmReadXPoseCntrl  -  control codel of CONTROL request ReadXPose
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmReadXPoseCntrl(int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmDefGoto_Angle_qi_PositionCntrl  -  control codel of CONTROL request DefGoto_Angle_qi_Position
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_DEF_GOTO_ANGLE_POSITION
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmDefGoto_Angle_qi_PositionCntrl(XARM_Gb_q6 *goalPoseq, int *report)
{
  SDI_F->matTrajExt.el[0][0] = goalPoseq->q1;
  SDI_F->matTrajExt.el[0][1] = goalPoseq->q2;
  SDI_F->matTrajExt.el[0][2] = goalPoseq->q3;
  SDI_F->matTrajExt.el[0][3] = goalPoseq->q4;
  SDI_F->matTrajExt.el[0][4] = goalPoseq->q5;
  SDI_F->matTrajExt.el[0][5] = goalPoseq->q6;  

  SDI_F->consignType = XARM_ANGLE_CONSIGN_TYPE;

  return OK;
}

/*------------------------------------------------------------------------
 * xarmDefGoto_XYZ_V_Angle_PositionCntrl  -  control codel of CONTROL request DefGoto_XYZ_V_Angle_Position
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_DEF_GOTO_XYZ_V_ANGLE_POSITION
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmDefGoto_XYZ_V_Angle_PositionCntrl(TP_ORIGIN_QUAT *goalPoseOriginQ, int *report)
{
  TP_QUAT QTO;
  double normaQ;
  TP_ORIGIN_QUAT QTO_1;

  QTO_1.x = goalPoseOriginQ->x;
  QTO_1.y = goalPoseOriginQ->y; 
  QTO_1.z = goalPoseOriginQ->z; 
  QTO_1.ex = goalPoseOriginQ->ex;
  QTO_1.ey = goalPoseOriginQ->ey;
  QTO_1.ez = goalPoseOriginQ->ez;
  QTO_1.angle = goalPoseOriginQ->angle;

  /* Position To = Current Position */
  SDI_F->matTrajQ.p.x = QTO_1.x;
  SDI_F->matTrajQ.p.y = QTO_1.y;
  SDI_F->matTrajQ.p.z = QTO_1.z;

  /* Quaternion To = Current Orientation */
  QTO.n = cos( (QTO_1.angle) / 2 );
  QTO.i = (QTO_1.ex) * sin((QTO_1.angle)/2);
  QTO.j = (QTO_1.ey) * sin((QTO_1.angle)/2);
  QTO.k = (QTO_1.ez) * sin((QTO_1.angle)/2);
  normaQ = QTO.n * QTO.n
    + QTO.i * QTO.i 
    + QTO.j * QTO.j
    + QTO.k * QTO.k;
  normaQ=sqrt(normaQ);
  if(normaQ==0) {
    PRINT_ERROR("DefGoto_XYZ_V_Angle_PositionCntrl - ERROR - Quaternion Goal Norm = 0");
    *report =  S_xarm_WRONG_VALUE; 
    return ERROR;
  }
  QTO.n = QTO.n/normaQ;
  QTO.i = QTO.i/normaQ;
  QTO.j = QTO.j/normaQ;
  QTO.k = QTO.k/normaQ;
  SDI_F->matTrajQ.q.n = QTO.n;
  SDI_F->matTrajQ.q.i = QTO.i;
  SDI_F->matTrajQ.q.j = QTO.j;
  SDI_F->matTrajQ.q.k = QTO.k;
  
  /* Init of Consign Type */
  SDI_F->consignType = XARM_QUATERNION_CONSIGN_TYPE;

  return OK;
}

/*------------------------------------------------------------------------
 * xarmDefGotoHandRelative_XYZ_V_Angle_PositionCntrl  -  control codel of CONTROL request DefGotoHandRelative_XYZ_V_Angle_Position
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_DEF_GOTO_HAND_RELATIVE_XYZ_V_ANGLE_POSITION
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmDefGotoHandRelative_XYZ_V_Angle_PositionCntrl(TP_ORIGIN_QUAT *goalPoseOriginQ, int *report)
{
  double normaQ;
  TP_ORIGIN_QUAT QTO_1;
  TP_POSE posRelTo;
  Gb_quat Gb_posRelTo;
  Gb_th Relth;
  Gb_th thTool;
  Gb_th Next_thTool;
  TP_QUAT GBQuatL;
  POSTER_ID posterQarmPos;
  QARM_POSITION_STR qarmPosition2;

  /* Degree to radian Conversion */
  QTO_1.ex = goalPoseOriginQ->ex;
  QTO_1.ey = goalPoseOriginQ->ey;
  QTO_1.ez = goalPoseOriginQ->ez;
  QTO_1.angle = goalPoseOriginQ->angle;

  /* Calcul and normalization of goal quaternion in the hand frame */
  posRelTo.p.x = goalPoseOriginQ->x ;
  posRelTo.p.y = goalPoseOriginQ->y;
  posRelTo.p.z = goalPoseOriginQ->z;
  posRelTo.q.n = cos( (QTO_1.angle) / 2 );
  posRelTo.q.i = (QTO_1.ex) * sin((QTO_1.angle)/2);
  posRelTo.q.j = (QTO_1.ey) * sin((QTO_1.angle)/2);
  posRelTo.q.k = (QTO_1.ez) * sin((QTO_1.angle)/2);

  normaQ = posRelTo.q.n * posRelTo.q.n
    + posRelTo.q.i * posRelTo.q.i 
    + posRelTo.q.j * posRelTo.q.j
    + posRelTo.q.k * posRelTo.q.k;
  normaQ=sqrt(normaQ);
  if(normaQ==0) {
    PRINT_ERROR("DefGotoHandRelative_XYZ_V_Angle_PositionCntrl - ERROR - Quaternion Goal Norm = 0");
    *report =  S_xarm_WRONG_VALUE; 
    return ERROR;
  }
  posRelTo.q.n = posRelTo.q.n/normaQ;
  posRelTo.q.i = posRelTo.q.i/normaQ;
  posRelTo.q.j = posRelTo.q.j/normaQ;
  posRelTo.q.k = posRelTo.q.k/normaQ;

  /* Transform the quaternion to the Gb quaterniom type */
  Gb_posRelTo.x =  posRelTo.p.x;
  Gb_posRelTo.y =  posRelTo.p.y;
  Gb_posRelTo.z =  posRelTo.p.z;
  Gb_posRelTo.w =  posRelTo.q.n; 
  Gb_posRelTo.vx =  posRelTo.q.i; 
  Gb_posRelTo.vy =  posRelTo.q.j; 
  Gb_posRelTo.vz =  posRelTo.q.k;
 
  /* Gb quaternion to matrix rotation */
  Gb_quat_th(&Gb_posRelTo, &Relth);

  /* INIT POSTERS */
  if (posterFind(QARM_POSITION_POSTER_NAME, &posterQarmPos) == ERROR) {
    PRINT_ERROR("Can not find POSTER POSITION");
    printf ("Can not find POSTER POSITION %s", QARM_POSITION_POSTER_NAME);
    *report =  S_xarm_POSTER_NOT_FOUND;
    return ETHER;
  }

 /* Read Poster Data and Status from QARM */
  if (xarmQARM_POSITION_STRPosterRead(posterQarmPos, &qarmPosition2)!=OK) {
    PRINT_ERROR("Can not read POSITION_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ERROR;
  }

  /* Init xarm R6/Rt */
  Gb_th_produit(&(qarmPosition2.th06), xarmR6Rt, &thTool); 
  thTool.vp.z += XARM_BASE_HEIGHT;  
  
  /* Calcul of goal Matrix rotation in the base frame */
  Gb_th_produit(&thTool, &Relth , &Next_thTool); 

  /* Calcul and normalization of goal quaternion in the base frame */ 
  if (RotMat2Quat(&Next_thTool, &GBQuatL) != OK) { 
    PRINT_ERROR("Pb - xarmDefGotoHandRelative_XYZ_V_Angle_PositionCntrl"); 
    return ERROR; 
  } 
  normaQ = GBQuatL.n * GBQuatL.n
    + GBQuatL.i * GBQuatL.i
    + GBQuatL.j * GBQuatL.j
    + GBQuatL.k * GBQuatL.k;
  normaQ=sqrt(normaQ);
  if(normaQ==0) {
    PRINT_ERROR("DefGotoHandRelative_XYZ_V_Angle_PositionCntrl - ERROR - Quaternion Goal Norm2 = 0");
    *report =  S_xarm_WRONG_VALUE; 
    return ERROR;
  }
  GBQuatL.n = GBQuatL.n /normaQ;
  GBQuatL.i = GBQuatL.i /normaQ;
  GBQuatL.j = GBQuatL.j /normaQ;
  GBQuatL.k = GBQuatL.k /normaQ;

  /* Update the goal position and orientation */
  /* Position To = Current Position */
  SDI_F->matTrajQ.p.x = Next_thTool.vp.x;
  SDI_F->matTrajQ.p.y = Next_thTool.vp.y;
  SDI_F->matTrajQ.p.z = Next_thTool.vp.z;
  /* Quaternion To = Current Orientation */
  SDI_F->matTrajQ.q.n = GBQuatL.n;
  SDI_F->matTrajQ.q.i = GBQuatL.i;
  SDI_F->matTrajQ.q.j = GBQuatL.j;
  SDI_F->matTrajQ.q.k = GBQuatL.k;
  
  /* Init of Consign Type */
  SDI_F->consignType = XARM_QUATERNION_CONSIGN_TYPE;

  return OK;
}

/*------------------------------------------------------------------------
 * xarmDefGotoXPoseCntrl  -  control codel of CONTROL request DefGotoXPose
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_DEF_GOTOXPOSE
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmDefGotoXPoseCntrl(TP_POSE *goalPose, int *report)
{  
  TP_QUAT QTO;
  double normaQ;

  /* Position To = Current Position */
  SDI_F->matTrajQ.p.x = goalPose->p.x;
  SDI_F->matTrajQ.p.y = goalPose->p.y;
  SDI_F->matTrajQ.p.z = goalPose->p.z;


  /* Quaternion To = Current Orientation */
  QTO.n = goalPose->q.n;
  QTO.i = goalPose->q.i;
  QTO.j = goalPose->q.j;
  QTO.k = goalPose->q.k;

  normaQ = QTO.n * QTO.n
    + QTO.i * QTO.i 
    + QTO.j * QTO.j
    + QTO.k * QTO.k;
  normaQ=sqrt(normaQ);
  if(normaQ==0) {
    PRINT_ERROR("DefGotoXPoseCntrl - ERROR - Quaternion Goal Norm = 0");
    *report =  S_xarm_WRONG_VALUE; 
    return ERROR;
  }

  QTO.n = QTO.n/normaQ;
  QTO.i = QTO.i/normaQ;
  QTO.j = QTO.j/normaQ;
  QTO.k = QTO.k/normaQ;

  SDI_F->matTrajQ.q.n = QTO.n;
  SDI_F->matTrajQ.q.i = QTO.i;
  SDI_F->matTrajQ.q.j = QTO.j;
  SDI_F->matTrajQ.q.k = QTO.k;

  /* Init of Consign Type */
  SDI_F->consignType = XARM_QUATERNION_CONSIGN_TYPE;

  return OK;
}

/*------------------------------------------------------------------------
 * xarmChangeModeCntrl  -  control codel of CONTROL request ChangeMode
 *
 * Description:    
 * Report: OK
 *                 S_xarm_CAN_NOT_CHANGE_MODE
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmChangeModeCntrl(XARM_ENUM_MODE *mode, int *report)
{

  return OK;
}

/*------------------------------------------------------------------------
 * xarmSetLimitsGotoCntrl  -  control codel of CONTROL request SetLimitsGoto
 *
 * Description:    
 * Report: OK
 *                 S_xarm_INCOMPATIBLE_LIMITS
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmSetLimitsGotoCntrl(TP_POSELIMITS *limitsGoto, int *report)
{
  double ljMax = limitsGoto->linear.maxJerk;
  double laMax = limitsGoto->linear.maxAcc;
  double lvMax = limitsGoto->linear.maxVel;
  double ajMax = limitsGoto->angular.maxJerk;
  double aaMax = limitsGoto->angular.maxAcc;
  double avMax = limitsGoto->angular.maxVel;
  if (ljMax <= 0 || laMax <= 0 || lvMax <= 0 ||
      ajMax <= 0 || aaMax <= 0 || avMax <= 0) {
    *report = S_xarm_INCOMPATIBLE_LIMITS;
    return ERROR;
  }
  printf("(laMax / ljMax) %f\n",(laMax / ljMax));
  printf(" 0.8 * (lvMax / laMax)%f\n",0.8 * (lvMax / laMax));


//  /* 20% margin to guarantee aMax is reach before vMax */
//  if ( (laMax / ljMax) > 0.8 * (lvMax / laMax) ) {
//    *report = S_xarm_INCOMPATIBLE_LIMITS;
//    return ERROR;
//  }
//  if ( (aaMax / ajMax) > 0.8 * (avMax / aaMax) ) {
//    *report = S_xarm_INCOMPATIBLE_LIMITS;
//    return ERROR;
//  }
//



  return OK;
}


/*------------------------------------------------------------------------
 * xarmPa10VelocitySetCntrl  -  control codel of CONTROL request Pa10VelocitySet
 *
 * Description:    
 * Report: OK
 *                 S_xarm_WRONG_VALUE
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmPa10VelocitySetCntrl(double *velocity, int *report)
{
  double aVel;
  aVel = *velocity *2.0/3.0;

  if (*velocity <= 0 || *velocity >= 0.2) {
    *report = S_xarm_INCOMPATIBLE_LIMITS;
    printf("xarm maxVelocity 0.2 m/s and minvelocity 0 m/s\n");
    return ERROR;
  }

  SDI_F->limitsGoto.linear.maxJerk = *velocity * 6;
  SDI_F->limitsGoto.linear.maxAcc = *velocity * 2;  
  SDI_F->limitsGoto.linear.maxVel = *velocity;
 SDI_F->limitsGoto.angular.maxJerk =  *velocity * 6;
 SDI_F->limitsGoto.angular.maxAcc = *velocity * 2; 
 SDI_F->limitsGoto.angular.maxVel = *velocity;
 // SDI_F->limitsGoto.angular.maxJerk = aVel *6;
 // SDI_F->limitsGoto.angular.maxAcc = aVel * 2;
 // SDI_F->limitsGoto.angular.maxVel = aVel;



  printf("   SDI_F->limitsGoto.linear.maxJerk %f \n", SDI_F->limitsGoto.linear.maxJerk);
  printf("   SDI_F->limitsGoto.linear.maxAcc %f \n", SDI_F->limitsGoto.linear.maxAcc);
  printf("   SDI_F->limitsGoto.linear.maxVel %f \n", SDI_F->limitsGoto.linear.maxVel);
  printf("   SDI_F->limitsGoto.angular.maxJerk %f \n", SDI_F->limitsGoto.angular.maxJerk);
  printf("   SDI_F->limitsGoto.angular.maxAcc %f \n", SDI_F->limitsGoto.angular.maxAcc);
  printf("   SDI_F->limitsGoto.angular.maxVel %f \n", SDI_F->limitsGoto.angular.maxVel);
  return OK;
}


/*------------------------------------------------------------------------
 * xarmSetLimitsExternCntrl  -  control codel of CONTROL request SetLimitsExtern
 *
 * Description:    
 * Report: OK
 *                 S_xarm_INCOMPATIBLE_LIMITS
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmSetLimitsExternCntrl(TP_POSELIMITS *limitsExtern, int *report)
{
  return xarmSetLimitsGotoCntrl(limitsExtern, report);
}

/*------------------------------------------------------------------------
 * xarmSetPIGainsCntrl  -  control codel of CONTROL request SetPIGains
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmSetPIGainsCntrl(XARM_PI_CONTROL_GAIN_STR *PIGain, int *report)
{
  SDI_F->pa10Control.PX.Kp = PIGain->Linear_Kp; 
  SDI_F->pa10Control.PX.Ti = PIGain->Linear_Ti;

  SDI_F->pa10Control.PY.Kp = PIGain->Linear_Kp;  
  SDI_F->pa10Control.PY.Ti = PIGain->Linear_Ti;  

  SDI_F->pa10Control.PZ.Kp = PIGain->Linear_Kp;  
  SDI_F->pa10Control.PZ.Ti = PIGain->Linear_Ti;  

  SDI_F->pa10Control.TX.Kp = PIGain->Angular_Kp;  
  SDI_F->pa10Control.TX.Ti = PIGain->Angular_Ti;  

  SDI_F->pa10Control.TY.Kp = PIGain->Angular_Kp;  
  SDI_F->pa10Control.TY.Ti = PIGain->Angular_Ti;  

  SDI_F->pa10Control.TZ.Kp = PIGain->Angular_Kp;  
  SDI_F->pa10Control.TZ.Ti = PIGain->Angular_Ti;  

  return OK;
}

/*------------------------------------------------------------------------
 * xarmReadTrajectoryCntrl  -  control codel of CONTROL request ReadTrajectoryFile
 *
 * Description:    
 * Report: OK
 *                 S_xarm_FILE_NOT_FOUND
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmReadTrajectoryCntrl(XARM_FILE_STR *fileTI, int *report)
{
  FILE *fileptr = NULL;
  int indexTrajectory = 0;
  int i = 0;
  float q1F, q2F, q3F, q4F, q5F,q6F;

  /* Open File Pointer */
  if ((fileptr = fopen(fileTI->file,"r"))==NULL) {
    PRINT_ERROR("ERROR - Open File TI");
    *report = S_xarm_FILE_NOT_FOUND;
    return ERROR;
  } 
  /* Init first value */
  SDI_F->matTrajExt.el[0][0] = 0;
  SDI_F->matTrajExt.el[0][1] = 0;
  SDI_F->matTrajExt.el[0][2] = 0;
  SDI_F->matTrajExt.el[0][3] = 0;
  SDI_F->matTrajExt.el[0][4] = 0;
  SDI_F->matTrajExt.el[0][5] = 0;
  
/* Read File Variables */
  while(!feof(fileptr)) {
    fscanf(fileptr, "%f\t %f\t%f\t%f\t%f\t%f\n",&q1F,&q2F,&q3F,&q4F,&q5F,&q6F);
    SDI_F->matTrajExt.el[indexTrajectory][0] = (double)q1F;
    SDI_F->matTrajExt.el[indexTrajectory][1] = (double)q2F;
    SDI_F->matTrajExt.el[indexTrajectory][2] = (double)q3F;
    SDI_F->matTrajExt.el[indexTrajectory][3] = (double)q4F;
    SDI_F->matTrajExt.el[indexTrajectory][4] = (double)q5F;
    SDI_F->matTrajExt.el[indexTrajectory][5] = (double)q6F;
    indexTrajectory++;
  }
  fclose(fileptr);

  for (i=0; i<indexTrajectory;i++) {
    fprintf(stdout,"Q0 %f, Q1 %f, Q2 %f, Q3 %f, Q4 %f, Q5 %f \n",SDI_F->matTrajExt.el[i][0],SDI_F->matTrajExt.el[i][1],SDI_F->matTrajExt.el[i][2],SDI_F->matTrajExt.el[i][3],SDI_F->matTrajExt.el[i][4],SDI_F->matTrajExt.el[i][5]);   
  } 
  SDI_F->matTrajExt.rows = indexTrajectory;
  fprintf(stdout, "TRAJECTORY FILE READED without SELECTION \n");
  fprintf(stdout, "Number of Points: %d \n", SDI_F->matTrajExt.rows);
  SDI_F->consignType = XARM_EXTERN_CONSIGN_TYPE;

  return OK;
}

/*------------------------------------------------------------------------
 * xarmReadTrajectoryArrayCntrl  -  control codel of CONTROL request ReadTrajectoryArray
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_SIZE
 *
 * Returns:    OK or ERROR
 */

/*STATUS
  xarmReadTrajectoryArrayCntrl(char posterName[200], int *report)
*/

STATUS 
xarmReadTrajectoryArrayCntrl(XARM_STRING *(posterName), int *report)
{
  int i,j;
  double dsq;
  TP_TRAJ AuxTraj;
  double d0,d1,d2,d3,d4,d5;



  /* Find Poster */
  if (posterFind(posterName->str, &posterPlanTraj) == ERROR) {
    PRINT_ERROR("Can not find POSTER TRAJECTORY");
    *report =  S_xarm_POSTER_NOT_FOUND;
    return ERROR;
  }

  /* Verify poster NON NULL */
  if (posterPlanTraj == NULL) {
    PRINT_ERROR("Poster of planned trajectroy NULL");
    *report = S_xarm_POSTER_NOT_FOUND;
    return ERROR;
  }

  /* Clean Global Variable for Poster */
  for (i=1; i<= XARM_MAX_NB_POSITIONS; i++) {
    planTraj.trajectory[0][i] = 0.0;
    planTraj.trajectory[1][i] = 0.0;
    planTraj.trajectory[2][i] = 0.0;
    planTraj.trajectory[3][i] = 0.0;
    planTraj.trajectory[4][i] = 0.0;
    planTraj.trajectory[5][i] = 0.0;
  }

  if (xarmXARM_TRAJECTORYPosterRead(posterPlanTraj, &planTraj)!=OK) {
    PRINT_ERROR("Can not read GP_SDI_STR_POSTER");
    *report =  S_xarm_POSTER_NOT_READ;
    return ERROR;
  }
  printf("planTraj.nb_positions %d\n",planTraj.nb_positions);
  /* Verify number of positions */
  if (planTraj.nb_positions == 0 ) {
    *report =  S_xarm_ERROR_SIZE;
    return OK;
  }


  /* Clean Global Variable for Poster */
  for (i=1; i<= planTraj.nb_positions; i++) {
    fprintf(stdout,"p %d  %f %f %f %f %f %f \n",
	    i, planTraj.trajectory[0][i],  planTraj.trajectory[1][i],  planTraj.trajectory[2][i], 
	       planTraj.trajectory[3][i],  planTraj.trajectory[4][i],  planTraj.trajectory[5][i]);
  }

  AuxTraj.rows = planTraj.nb_positions;
  for (i=1; i<= planTraj.nb_positions;i++) {
    AuxTraj.el[i-1][0] = planTraj.trajectory[0][i];
    AuxTraj.el[i-1][1] = planTraj.trajectory[1][i];
    AuxTraj.el[i-1][2] = planTraj.trajectory[2][i];
    AuxTraj.el[i-1][3] = planTraj.trajectory[3][i];
    AuxTraj.el[i-1][4] = planTraj.trajectory[4][i];
    AuxTraj.el[i-1][5] = planTraj.trajectory[5][i];
  }

  /*Select long steps */
  SDI_F->matTrajExt.el[0][0] = AuxTraj.el[0][0];
  SDI_F->matTrajExt.el[0][1] = AuxTraj.el[0][1];
  SDI_F->matTrajExt.el[0][2] = AuxTraj.el[0][2];
  SDI_F->matTrajExt.el[0][3] = AuxTraj.el[0][3];
  SDI_F->matTrajExt.el[0][4] = AuxTraj.el[0][4];
  SDI_F->matTrajExt.el[0][5] = AuxTraj.el[0][5];
  j=0;
  for (i=1; i<AuxTraj.rows; i++) {
    d0 = SDI_F->matTrajExt.el[j][0] - AuxTraj.el[i][0];
    d1 = SDI_F->matTrajExt.el[j][1] - AuxTraj.el[i][1];
    d2 = SDI_F->matTrajExt.el[j][2] - AuxTraj.el[i][2];
    d3 = SDI_F->matTrajExt.el[j][3] - AuxTraj.el[i][3];
    d4 = SDI_F->matTrajExt.el[j][4] - AuxTraj.el[i][4];
    d5 = SDI_F->matTrajExt.el[j][5] - AuxTraj.el[i][5];
    dsq = d0*d0 + d1*d1 + d2*d2 + d3*d3 + d4*d4 + d5*d5;

    if (dsq>0.001) {
      j = j+1; 
      SDI_F->matTrajExt.el[j][0] = AuxTraj.el[i][0];
      SDI_F->matTrajExt.el[j][1] = AuxTraj.el[i][1];
      SDI_F->matTrajExt.el[j][2] = AuxTraj.el[i][2];
      SDI_F->matTrajExt.el[j][3] = AuxTraj.el[i][3];
      SDI_F->matTrajExt.el[j][4] = AuxTraj.el[i][4];
      SDI_F->matTrajExt.el[j][5] = AuxTraj.el[i][5];
    }
  }

  SDI_F->matTrajExt.rows = j+1;
  
  /* To guarantee the last position */
  d0 = SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][0] - AuxTraj.el[AuxTraj.rows -1][0]; 
  d1 = SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][1] - AuxTraj.el[AuxTraj.rows -1][1]; 
  d2 = SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][2] - AuxTraj.el[AuxTraj.rows -1][2]; 
  d3 = SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][3] - AuxTraj.el[AuxTraj.rows -1][3]; 
  d4 = SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][4] - AuxTraj.el[AuxTraj.rows -1][4]; 
  d5 = SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][5] - AuxTraj.el[AuxTraj.rows -1][5]; 
  dsq = d0 + d1 +d2 + d3 + d4 + d5;
  if (dsq!= 0) {
    SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][0] = AuxTraj.el[AuxTraj.rows -1][0]; 
    SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][1] = AuxTraj.el[AuxTraj.rows -1][1]; 
    SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][2] = AuxTraj.el[AuxTraj.rows -1][2]; 
    SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][3] = AuxTraj.el[AuxTraj.rows -1][3]; 
    SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][4] = AuxTraj.el[AuxTraj.rows -1][4]; 
    SDI_F->matTrajExt.el[SDI_F->matTrajExt.rows -1][5] = AuxTraj.el[AuxTraj.rows -1][5];
    fprintf(stdout, "Last configuration correction \n");
  }


/*   for (i=0; i<=Traj.nb_positions;i++) {  */
/*     fprintf(stdout,"T0 %f, T1 %f, T2 %f, T3 %f, T4 %f, T5 %f \n",Traj.traj[0][i],Traj.traj[1][i],Traj.traj[2][i],Traj.traj[3][i],Traj.traj[4][i],Traj.traj[5][i]);  */
/*    }  */
/*   fprintf(stdout,"\n"); */
  
/*   for (i=0; i<AuxTraj.rows;i++) {  */
/*      fprintf(stdout,"A0 %f, A1 %f, A2 %f, A3 %f, A4 %f, A5 %f \n",AuxTraj.el[i][0],AuxTraj.el[i][1],AuxTraj.el[i][2],AuxTraj.el[i][3],AuxTraj.el[i][4],AuxTraj.el[i][5]);  */
/*    }  */
/*   fprintf(stdout,"\n"); */
  for (i=0;i<SDI_F->matTrajExt.rows ;i++) { 
     fprintf(stdout,"%1.3f, %1.3f, %1.3f, %1.3f, %1.3f, %1.3f \n",SDI_F->matTrajExt.el[i][0],SDI_F->matTrajExt.el[i][1],SDI_F->matTrajExt.el[i][2],SDI_F->matTrajExt.el[i][3],SDI_F->matTrajExt.el[i][4],SDI_F->matTrajExt.el[i][5]); 
   } 
  fprintf(stdout,"\n");
  fprintf(stdout, "TRAJECTORY ARRAY READED \n");

  /* Free Poster ID */
  posterPlanTraj = NULL;

  SDI_F->consignType = XARM_EXTERN_CONSIGN_TYPE;
  
  return OK;
}

/*------------------------------------------------------------------------
 * xarmTrackingTrajectoryCntrl  -  control codel of EXEC request TrackingTrajectory
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_TP
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmTrackingTrajectoryCntrl(int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmTrackingTrajectory_V2Cntrl  -  control codel of EXEC request TrackingTrajectory_V2
 *
 * Description:    
 * Report: OK
 *                 S_xarm_ERROR_TP
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmTrackingTrajectory_V2Cntrl(int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmReadMotionStateCntrl  -  control codel of CONTROL request ReadMotionState
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmReadMotionStateCntrl(int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmInitAcqDataCntrl  -  control codel of CONTROL request InitAcqData
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmInitAcqDataCntrl(int *report)
{
  SDI_F->inDataEtat = XARM_INIT;
  return OK;
}

/*------------------------------------------------------------------------
 * xarmStopAcqDataCntrl  -  control codel of CONTROL request StopAcqData
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmStopAcqDataCntrl(int *report)
{
  SDI_F->inDataEtat = XARM_OFF;
  return OK;
}

/*------------------------------------------------------------------------
 * xarmExternalTrajectoryCntrl  -  control codel of EXEC request ExternalTrajectory
 *
 * Description:    
 * Report: OK
 *                 S_xarm_POSTER_NOT_FOUND
 *
 * Returns:    OK or ERROR
 */


/* STATUS */
/* xarmExternalTrajectoryCntrl(char posterName[200], int *report) */
STATUS 
xarmExternalTrajectoryCntrl(XARM_STRING *(posterName), int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmExternalConfigurationCntrl  -  control codel of EXEC request ExternalConfiguration
 *
 * Description:    
 * Report: OK
 *                 S_xarm_POSTER_NOT_FOUND
 *
 * Returns:    OK or ERROR
 */

/* STATUS */
/* xarmExternalConfigurationCntrl(char posterName[200], int *report) */
STATUS 
xarmExternalConfigurationCntrl(XARM_STRING *(posterName), int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmExternalDrivingCntrl  -  control codel of EXEC request ExternalDriving
 *
 * Description:    
 * Report: OK
 *                 S_xarm_POSTER_NOT_FOUND
 *
 * Returns:    OK or ERROR
 */

/* STATUS */
/* xarmExternalDrivingCntrl(char posterName[200], int *report) */
STATUS 
xarmExternalDrivingCntrl(XARM_STRING *(posterName), int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmStopExternalTrajectoryCntrl  -  control codel of CONTROL request StopExternalTrajectory
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmStopExternalTrajectoryCntrl(int *report)
{
  SDI_F->stopExternalDrive = XARM_ON;
  SDI_F->inDataEtat = XARM_OFF;
  return OK;
}

/*------------------------------------------------------------------------
 * xarmStopCntrl  -  control codel of CONTROL request Stop
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmStopCntrl(int *report)
{
  SDI_F->mode = XARM_STOP;
  SDI_F->xarmTrackSegment = XARM_TRACK_OFF;
 /* Initialize queues */
  queueInit(&(SDI_F->Seg[0]), "SegX");
  queueInit(&(SDI_F->Seg[1]), "SegY");
  queueInit(&(SDI_F->Seg[2]), "SegZ");
  queueInit(&(SDI_F->Seg[3]), "SegQN");
  queueInit(&(SDI_F->Seg[4]), "SegQI");
  queueInit(&(SDI_F->Seg[5]), "SegQJ");
  queueInit(&(SDI_F->Seg[6]), "SegQK");
   /* qVel */
  SDI_F->pa10Arm.qVel.q1 = 0;
  SDI_F->pa10Arm.qVel.q2 = 0;
  SDI_F->pa10Arm.qVel.q3 = 0;
  SDI_F->pa10Arm.qVel.q4 = 0;
  SDI_F->pa10Arm.qVel.q5 = 0;
  SDI_F->pa10Arm.qVel.q6 = 0;
  SDI_F->consignType = XARM_NOT_SELECTED_CONSIGN_TYPE;

  return OK;
}

/*------------------------------------------------------------------------
 * xarmSetPosterPoseCntrl  -  control codel of CONTROL request SetPosterPose
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

/* STATUS */
/* xarmSetPosterPoseCntrl(char posterName[200], int *report) */
STATUS 
xarmSetPosterPoseCntrl(XARM_STRING *(posterName), int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * xarmMGDCntrl  -  control codel of CONTROL request MGD
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmMGDCntrl(XARM_Gb_q6 *mgdQ6, int *report)
{
  Gb_q6 q6;

  q6.q1 = mgdQ6->q1;
  q6.q2 = mgdQ6->q2;
  q6.q3 = mgdQ6->q3;
  q6.q4 = mgdQ6->q4;
  q6.q5 = mgdQ6->q5;
  q6.q6 = mgdQ6->q6;
  xarmMGDQuat(&q6, &(SDI_F->mgdQuat));
  return OK;
}

/*------------------------------------------------------------------------
 * xarmMGICntrl  -  control codel of CONTROL request MGI
 *
 * Description:    
 * Report: OK
 *                 S_xarm_approximate_result
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmMGICntrl(TP_POSE *mgdQuat, int *report)
{
  /* ... add your code here ... */
  return OK;
}







/*------------------------------------------------------------------------
 * xarmStopExternalTrackingCntrl  -  control codel of CONTROL request StopExternalTracking
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmStopExternalTrackingCntrl(int *report)
{
  SDI_F->mode = XARM_STOP;
  return OK;
}





/*------------------------------------------------------------------------
 * xarmRealTimeTrackPoseCntrl  -  control codel of EXEC request RealTimeTrackPose
 *
 * Description:    
 * Report: OK
 *                 S_xarm_POSTER_NOT_FOUND
 *                 S_xarm_POSTER_NOT_READ
 *                 S_xarm_ERROR_TP
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmRealTimeTrackPoseCntrl(XARM_STRING *posPosterName, int *report)
{
  /* ... add your code here ... */
  return OK;
}


/*------------------------------------------------------------------------
 * xarmSetPrintDataCntrl  -  control codel of CONTROL request SetPrintData
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmSetPrintDataCntrl(int *printData, int *report)
{
  if (*printData !=0 && *printData!=1) {
    PRINT_ERROR("printData error value\n");
    return ERROR;
  }
  SDI_F->printData = *printData;
  return OK;
}


/*------------------------------------------------------------------------
 * xarmSetSaveFilesCntrl  -  control codel of CONTROL request SetSaveFiles
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmSetSaveFilesCntrl(int *saveFiles, int *report)
{
 
  if (*saveFiles !=0 && *saveFiles!=1) {
    PRINT_ERROR("saveFiles error value\n");
    return ERROR;
  }
  SDI_F->saveFiles = *saveFiles;
  return OK;
}


/*------------------------------------------------------------------------
 * xarmSweetStopCntrl  -  control codel of CONTROL request SweetStop
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
xarmSweetStopCntrl(int *report)
{
  SDI_F->SweetStop = 1;
  return OK;
}


