/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** xarmFunctions.h
 **
 ** Codels called by execution task xarmMotion
 **
 ** Author: 
 ** Date: 
 **
 **/

#ifndef XARMFUNCTION_H 
#define XARMFUNCTION_H 

#include <portLib.h>
#include <stdlib.h>
#include "server/xarmHeader.h"
#include "server/qarmPosterLib.h"
#include <math.h>
#include "xarmStruct.h" 
#include "qarmConst.h"
#include "xarmConst.h"
#include "tpStruct.h"
#include "tpConst.h"
#undef GB_GENOM
#include <gb.h>
#include "Proto_gbModeles.h"
#include "Proto_gb.h"





#define PRINT_ERROR(Str)         fprintf(stderr, "%s: %d: %s \n", __FILE__, __LINE__,Str);
#define PRINT_DEBUG(Params,Var)  fprintf(stdout,Params,Var);

extern STATUS InitParamsPa10ControlC1(XARM_PI_CONTROL_GAIN_STR* piGain, XARM_POSE_CONTROL_STR *pa10Control);

extern STATUS InitParamsPa10Control(XARM_PI_CONTROL_GAIN_STR* piGain, TP_POSELIMITS *limitsGoto, XARM_POSE_CONTROL_STR *pa10Control);

extern STATUS InitParamsPa10ControlC2( XARM_POSE_CONTROL_STR *pa10Control );

extern STATUS InitControlsPa10(XARM_POSE_CONTROL_STR *pa10Control);

extern STATUS CalculLawOfControl(XARM_DIM_CONTROL_STR *dimControl, double ErrorDim);

extern XARM_STATUS InitxarmR6Rt(double* r0,double* r6,double* r7,XARM_THETA_STR* r8z);

extern void initPa10Parameters();

extern int xarmTh06_Tool(Gb_th* th06, Gb_th* thTool, TP_POSE* poseTool);

extern int xarmMGDQuat(Gb_q6* q6, TP_POSE* pose);

extern XARM_STATUS CalculOfJacobianMatrix(dmat Jac, Gb_jac Jac06, double r6, double r7); 

extern XARM_STATUS xarmTestSingularity(QARM_POSITION_STR* qarmPosition, TP_POSE* pose);

extern XARM_STATUS xarmTestReachable(QARM_POSITION_STR* qarmPosition, TP_POSE* pose);

extern XARM_STATUS xarmTestReachableBox(TP_POSE* pose) ;

extern XARM_STATUS xarmCalculOfAccVelPos(int tick, SEGMENT* seg, TP_VECTOR* v);

extern void VerifyAllQueue(QUEUE* SegX, QUEUE* SegY, QUEUE* SegZ, QUEUE* SegQN, QUEUE* SegQI, QUEUE* SegQJ, QUEUE* SegQK, int* VerifyIfItsLastSegment);

extern XARM_STATUS GeometricalCalculations( dmat PFrom, dmat PTo,dmat QFrom, dmat QTo,
				     double *MaxLDist,  double *MaxADist, dmat Dist,
				     dmat LDist, dmat AbsLDist, dmat ADist, dmat AbsADist, dmat AbsDist,
					    dmat Dir, int *report) ;

extern XARM_STATUS CalculPointToPointJerkProfile(dmat PFrom, dmat QFrom, TP_POSELIMITS Limits, double MaxLDist, double MaxADist, double MaxDist,
						 dmat AbsDist, dmat Dir, TP_JERKS jerk[], TP_TIMES Acc[], TP_TIMES Vel[], TP_TIMES Pos[], TP_TIMES* TNE, int* report);

extern XARM_STATUS CalculTransitionWithStopMotion(double PosStop[], TP_POSELIMITS limitsGoto, TP_COND INITIAL_COND[], TP_TIMES Acc[], TP_TIMES Vel[], TP_TIMES Pos[],
						  double OptimalTransitionTimesWithStop[], TP_AXIS_TIMES *AxisTimes_End, TP_AXIS_TIMES *AxisTimes_Start, 
						  int DirTransition_End [], int DirTransition_Start [], TP_TIMES Acc_End[], TP_TIMES Vel_End[], TP_TIMES Pos_End[],
						  TP_TIMES Acc_Start[], TP_TIMES Vel_Start[], TP_TIMES Pos_Start[], int* report);


extern XARM_STATUS CalculPointToPointJerkProfileV2(dmat PFrom, dmat QFrom, TP_POSELIMITS Limits, double MaxLDist, double MaxADist, double MaxDist, dmat AbsDist, dmat Dir, TP_MOTION* motion, int* report);

extern XARM_STATUS FindMaxTimeAxis(TP_POSELIMITS limitsGoto, TP_COND INITIAL_COND[], TP_TIMES Acc[], TP_TIMES Vel[], TP_TIMES Pos[], double *MaxTime, int* MaxTimeAxis);

#endif 
