/*
 * Copyright (c) 2006-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** xarmPlanificationCodels.c
 **
 ** Codels called by execution task xarmPlanification
 **
 ** Author: Xavier BROQUERE
 ** Date: October 2007
 **
 **/

#include <portLib.h>
#include <stdlib.h>
#include "stdio.h"
#include "server/xarmHeader.h"
#include <math.h>
#include "server/qarmPosterLib.h"
#include "xarmConst.h"
#include "tpConst.h"
#include "Proto_gb.h"
#include "Proto_gbModeles.h"
#include "libeuler/pomEuler.h"
#include "xarmFunctions.h"
#include "tpFunctions.h"
#include "cFunctions.h"
#include "xarmJerkprofile.h"
//#include "xarmJerkprofile.c"
#include "queue.h"

static dmat PFrom;                               /* Position From */
static dmat PTo;                                 /* Position To   */
static dmat QFrom;                               /* Quaternion From */
static dmat QTo;                                 /* Quaternion To */
static dmat Dist;                                /* Distance = To - From */
static dmat AbsDist;                             /* Distance Magnitud */
static dmat Dir;                                 /* Motion Direction */
static dmat ADist;                               /* Initial Angular Distance */ 
static dmat LDist;                               /* Initial Linear Distance */ 
static dmat AbsADist;                            /* Initial Absolute Angular Distance */ 
static dmat AbsLDist;                            /* Initial Absolute Linear Distance */ 
static TP_MAT matPoseTI;
static int indexTrajectory;
static XARM_SOIL soil;
static int timeSX, timeSY, timeSZ, timeSQN, timeSQI, timeSQJ, timeSQK;
static TP_COND INITIAL_COND[7]; 


FILE *fileptr;

/*------------------------------------------------------------------------
 *
 * xarmPlanificationEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
xarmPlanificationEnd(void)
{
  return OK;
}

/*------------------------------------------------------------------------
 *
 * xarmPlanificationPerm  --  1st codel of permanent activity (called BEFORE other activities)
 *
 * Description: 
 * 
 * Reports:      OK
 *              S_xarm_ERROR_MODE
 *              S_xarm_POSTER_NOT_FOUND
 *              S_xarm_POSTER_NOT_READ
 *              S_xarm_WRONG_VALUE
 *              S_xarm_ERROR_TP
 * 
 * Returns:    OK or ERROR
 */

STATUS
xarmPlanificationPerm(int *report)
{
  /* ... add your code here ... */
  return OK;
}

#define NEWDMAT_IS_OK(testError, mess, m_report)		  \
  do {								  \
    if (testError) {						  \
      PRINT_ERROR(mess);					  \
      *report = m_report;					  \
      return ETHER;						  \
    }								  \
  } while(0)							  \


/*------------------------------------------------------------------------
 * PlanningTrajectory
 *
 * Description: 
 *
 * Reports:      OK
 *              S_xarm_ERROR_TP
 *              S_xarm_NOT_REACHABLE
 */

/* xarmPlanningTrajectoryStart  -  codel START of PlanningTrajectory
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmPlanningTrajectoryStart(int *report)
{

  TP_POSE mgdpose;
  Gb_q6 qL;
  double Jmax, Amax, Vmax;
  int i=0;
  int errnoMatDef;                          /* Error Auxiliar Variable */
  double MaxLDist;                          /* Maximum Linear Distance */
  double MaxADist;                          /* Maximum Angular Distance */
  double MaxDist;                           /* Maximum Distance */
  TP_TIMES TNE;
  TP_TIMES Acc[7], Vel[7], Pos[7];
  TP_JERKS jerk[7];

  extern int pa10Parameters;

  /* Initial PFrom and PTo */
  PFrom = newdmat(0,0,0,XARM_POS_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "QTo Definition", S_xarm_ERROR_TP);
  PTo = newdmat(0,0,0,XARM_POS_DIM-1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "PTo Definition", S_xarm_ERROR_TP);
  /* Initial QFrom and QTo */
  QFrom = newdmat(0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "QFrom Definition", S_xarm_ERROR_TP);
  QTo = newdmat(0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "QTo Definition", S_xarm_ERROR_TP);
  /* Distance Vector Definition */
  Dist = newdmat (0,0,0,XARM_NB_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Dist Definition", S_xarm_ERROR_TP);
  /* Absolute Distance Vector Definition */
  AbsDist = newdmat (0,0,0,XARM_NB_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "AbsDist Definition", S_xarm_ERROR_TP);
  /* Dir Vector Definition */
  Dir = newdmat (0,0,0,XARM_NB_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Dir Definition", S_xarm_ERROR_TP);
  /* Linear Distance Vector Definition */
  LDist = newdmat (0,0,0,XARM_POS_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Linear Distance Definition ", S_xarm_ERROR_TP);
  /* Absolute Linear Distance Vector Definition */
  AbsLDist = newdmat (0,0,0,XARM_POS_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Absolute Linear Distance Definition", S_xarm_ERROR_TP);
  /* Angular Distance Vector Definition */
  ADist = newdmat (0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Angular Distance Definition", S_xarm_ERROR_TP);
  /* Absolute Distance Vector Definition */
  AbsADist = newdmat (0,0,0,XARM_QUAT_DIM - 1,&errnoMatDef);
  NEWDMAT_IS_OK(errnoMatDef, "Absolute Angular Distance Definition", S_xarm_ERROR_TP);

   /* VERIFY IF CONSIGN REQUEST IS DONE */
  if (SDI_F->consignType == XARM_NOT_SELECTED_CONSIGN_TYPE) {
     PRINT_ERROR(" --- ERROR : NOT_SELECTED_CONSIGN_TYPE --- \n");  
     return ETHER;
  }

  /* Open a file to save initial and final conditions for transition motions */
  if(SDI_F->saveFiles == 1) {
    if ((fileptr = fopen("RefTransition.dat","ab"))==NULL) {
      PRINT_ERROR("CANNOT Open File RefTransition.dat ");
      SDI_F->saveFiles = 0;
    }
    //   fprintf(fileptr,"New Motion\n");
    // fprintf(fileptr,"Axis A0 V0 X0 Af Vf Xf \n");
  }

  /* INIT CONST AND VERIFY VALUES */
  /* Initialize queues */
  queueInit(&(SDI_F->Seg[0]), "SegX");
  queueInit(&(SDI_F->Seg[1]), "SegY");
  queueInit(&(SDI_F->Seg[2]), "SegZ");
  queueInit(&(SDI_F->Seg[3]), "SegQN");
  queueInit(&(SDI_F->Seg[4]), "SegQI");
  queueInit(&(SDI_F->Seg[5]), "SegQJ");
  queueInit(&(SDI_F->Seg[6]), "SegQK"); 

  soil.angularVelocity = 2*(SDI_F->limitsGoto.angular.maxJerk)*SAMPLING_TIME*SAMPLING_TIME;
  soil.linearVelocity  = 2*(SDI_F->limitsGoto.linear.maxJerk)*SAMPLING_TIME*SAMPLING_TIME;
  soil.angularPosition = 3*(SDI_F->limitsGoto.angular.maxJerk)*SAMPLING_TIME*SAMPLING_TIME*SAMPLING_TIME;
  soil.linearPosition  = 3*(SDI_F->limitsGoto.linear.maxJerk)*SAMPLING_TIME*SAMPLING_TIME*SAMPLING_TIME;

  matPoseTI.rows = 0;
  
  SDI_F->pa10Arm.Limits.linear.maxJerk = SDI_F->limitsGoto.linear.maxJerk;
  SDI_F->pa10Arm.Limits.linear.maxAcc  = SDI_F->limitsGoto.linear.maxAcc;
  SDI_F->pa10Arm.Limits.linear.maxVel  = SDI_F->limitsGoto.linear.maxVel;

  Jmax =  SDI_F->pa10Arm.Limits.linear.maxJerk;
  Amax =  SDI_F->pa10Arm.Limits.linear.maxAcc;
  Vmax =  SDI_F->pa10Arm.Limits.linear.maxVel;

  if (Jmax<= 0 || Amax<= 0 || Vmax <= 0) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Negatives or Zero - Linear");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  } 
  
  if (Amax/Jmax  > Vmax/Amax) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Values - Linear");
    *report =  S_xarm_WRONG_VALUE; 
    return ETHER;
  }

  SDI_F->pa10Arm.Limits.angular.maxJerk =  SDI_F->limitsGoto.angular.maxJerk;
  SDI_F->pa10Arm.Limits.angular.maxAcc  =  SDI_F->limitsGoto.angular.maxAcc;
  SDI_F->pa10Arm.Limits.angular.maxVel  =  SDI_F->limitsGoto.angular.maxVel;
  
  Jmax =  SDI_F->pa10Arm.Limits.angular.maxJerk;
  Amax =  SDI_F->pa10Arm.Limits.angular.maxAcc;
  Vmax =  SDI_F->pa10Arm.Limits.angular.maxVel;
  
  if (Jmax<= 0 || Amax<= 0 || Vmax <= 0) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Negatives or Zero - Angular");
    *report =  S_xarm_WRONG_VALUE; 
    return XARM_ERROR;
  } 
  if (Amax/Jmax  > Vmax/Amax) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - Jmax, Amax, Vmax Values - Angular");
    *report =  S_xarm_WRONG_VALUE; 
    return XARM_ERROR;
  }
  if (pa10Parameters == 0) {
    PRINT_ERROR("TrackingTrajectoryStart - ERROR - pa10Parameters is NULL");
    *report =  S_xarm_WRONG_VALUE; 
    return XARM_ERROR;
  }

  switch (SDI_F->consignType) {
    
  case XARM_QUATERNION_CONSIGN_TYPE  :
    /* INIT K FOR SR_INVERSE */
    SDI_F->pa10Arm.K_WEIGHT = XARM_K1_WEIGHT;
    matPoseTI.cols = XARM_NB_DIM;
    matPoseTI.rows = 1; 
    matPoseTI.el[0][0] = SDI_F->matTrajQ.p.x;
    matPoseTI.el[0][1] = SDI_F->matTrajQ.p.y;
    matPoseTI.el[0][2] = SDI_F->matTrajQ.p.z;
    matPoseTI.el[0][3] = SDI_F->matTrajQ.q.n;
    matPoseTI.el[0][4] = SDI_F->matTrajQ.q.i;
    matPoseTI.el[0][5] = SDI_F->matTrajQ.q.j;
    matPoseTI.el[0][6] = SDI_F->matTrajQ.q.k;
    if (xarmTestSingularity(&(SDI_F->pa10Arm.qarmPosition), &(SDI_F->matTrajQ)) != XARM_OK) {
      return ETHER;
    }
    //if(xarmTestReachable(&(SDI_F->pa10Arm.qarmPosition), &(SDI_F->matTrajQ)) != XARM_OK) {
    //  *report = S_xarm_NOT_REACHABLE;
    //  return ETHER;
    //}
    //if(xarmTestReachableBox(&(SDI_F->matTrajQ)) != XARM_OK) {
    //  *report = S_xarm_NOT_REACHABLE;
    //  return ETHER;
    //}
    printf("SWITCH XARM_QUATERNION_CONSIGN_TYPE OK \n");
    break;

  case XARM_ANGLE_CONSIGN_TYPE :
    /* INIT K FOR SR_INVERSE */
    SDI_F->pa10Arm.K_WEIGHT = XARM_K1_WEIGHT;
    matPoseTI.cols = XARM_NB_DIM;
    matPoseTI.rows = 1; 
    qL.q1= SDI_F->matTrajExt.el[0][0];
    qL.q2= SDI_F->matTrajExt.el[0][1];
    qL.q3= SDI_F->matTrajExt.el[0][2];
    qL.q4= SDI_F->matTrajExt.el[0][3];
    qL.q5= SDI_F->matTrajExt.el[0][4];
    qL.q6= SDI_F->matTrajExt.el[0][5];
    xarmMGDQuat(&qL, &mgdpose);
    matPoseTI.el[0][0] = mgdpose.p.x;
    matPoseTI.el[0][1] = mgdpose.p.y;
    matPoseTI.el[0][2] = mgdpose.p.z;
    matPoseTI.el[0][3] = mgdpose.q.n;
    matPoseTI.el[0][4] = mgdpose.q.i;
    matPoseTI.el[0][5] = mgdpose.q.j;
    matPoseTI.el[0][6] = mgdpose.q.k;
    if (xarmTestSingularity(&(SDI_F->pa10Arm.qarmPosition), &mgdpose) != XARM_OK) {
      return ETHER;
    }
    printf("SWITCH XARM_ANGLE_CONSIGN_TYPE OK \n");
    break;

  case XARM_EXTERN_CONSIGN_TYPE :
    /* INIT K FOR SR_INVERSE */
    SDI_F->pa10Arm.K_WEIGHT = XARM_K2_WEIGHT;
    matPoseTI.cols = XARM_NB_DIM;
    matPoseTI.rows =  SDI_F->matTrajExt.rows ; 
    for(i=0;i< SDI_F->matTrajExt.rows;i++) {
      qL.q1= SDI_F->matTrajExt.el[i][0];
      qL.q2= SDI_F->matTrajExt.el[i][1];
      qL.q3= SDI_F->matTrajExt.el[i][2];
      qL.q4= SDI_F->matTrajExt.el[i][3];
      qL.q5= SDI_F->matTrajExt.el[i][4];
      qL.q6= SDI_F->matTrajExt.el[i][5];
      //  printf(" %f %f %f %f %f %f \n", qL.q1, qL.q2 ,qL.q3 ,qL.q4 ,qL.q5 ,qL.q6);
      xarmMGDQuat(&qL, &mgdpose);
      printf(" %3.3f %3.3f %3.3f %3.3f %3.3f %3.3f %3.3f\n", mgdpose.p.x, mgdpose.p.y, mgdpose.p.z, mgdpose.q.n, mgdpose.q.i, mgdpose.q.j, mgdpose.q.k);
      matPoseTI.el[i][0] = mgdpose.p.x;
      matPoseTI.el[i][1] = mgdpose.p.y;
      matPoseTI.el[i][2] = mgdpose.p.z;
      matPoseTI.el[i][3] = mgdpose.q.n;
      matPoseTI.el[i][4] = mgdpose.q.i;
      matPoseTI.el[i][5] = mgdpose.q.j;
      matPoseTI.el[i][6] = mgdpose.q.k;
    }
    printf("SWITCH XARM_EXTERN_CONSIGN_TYPE OK\n");
    break;

  default :
    PRINT_ERROR("ERROR Type of Consign : XARM_NOT_SELECTED_CONSIGN_TYPE \n");
    return ETHER;
  }

  /*=====================================================*/
  /*                 First Trajectory                    */
  /*=====================================================*/ 
  
  /*Init From and To */
  /* Position From = Current Position */
  PFrom.el[0][0] = SDI_F->pose.p.x;
  PFrom.el[0][1] = SDI_F->pose.p.y;
  PFrom.el[0][2] = SDI_F->pose.p.z;
  /* Quaternion From = Current Orientation */
  QFrom.el[0][0] = SDI_F->pose.q.n;
  QFrom.el[0][1] = SDI_F->pose.q.i;  
  QFrom.el[0][2] = SDI_F->pose.q.j;
  QFrom.el[0][3] = SDI_F->pose.q.k;
  
  /* Position To = Goal Position 1 */
  PTo.el[0][0] = matPoseTI.el[0][0];
  PTo.el[0][1] = matPoseTI.el[0][1];
  PTo.el[0][2] = matPoseTI.el[0][2];
  /* Quaternion To = Goal Orientation 1 */
  QTo.el[0][0] = matPoseTI.el[0][3];
  QTo.el[0][1] = matPoseTI.el[0][4];
  QTo.el[0][2] = matPoseTI.el[0][5];
  QTo.el[0][3] = matPoseTI.el[0][6];

  if(GeometricalCalculations(PFrom, PTo, QFrom, QTo, &MaxLDist, &MaxADist, Dist, LDist, AbsLDist, ADist, AbsADist, AbsDist, Dir, report)!=OK) {
    PRINT_ERROR("xarm ERROR GeometricalCalculations\n");
    return ETHER;
  }
  CalculPointToPointJerkProfile(PFrom, QFrom, SDI_F->pa10Arm.Limits, MaxLDist, MaxADist, MaxDist, AbsDist, Dir, jerk, Acc, Vel, Pos, &TNE, report);

  /* Increment poster */
  SDI_F->dq.counter += 1;
  
  queueAddDetails(&(SDI_F->Seg[0]), 1, TNE.Tjpa, jerk[0].J1, 0, 0, PFrom.el[0][0], Dir.el[0][0]);
  queueAddDetails(&(SDI_F->Seg[0]), 2, TNE.Taca, jerk[0].J1, Acc[0].Tjpa, Vel[0].Tjpa, Pos[0].Tjpa, Dir.el[0][0]);
  queueAddDetails(&(SDI_F->Seg[0]), 3, TNE.Tjna, jerk[0].J1, Acc[0].Taca, Vel[0].Taca, Pos[0].Taca, Dir.el[0][0]);
  queueAddDetails(&(SDI_F->Seg[0]), 4, TNE.Tvc,  jerk[0].J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna, Dir.el[0][0]);
  queueAddDetails(&(SDI_F->Seg[1]), 1, TNE.Tjpa, jerk[1].J1, 0, 0, PFrom.el[0][1], Dir.el[0][1]);
  queueAddDetails(&(SDI_F->Seg[1]), 2, TNE.Taca, jerk[1].J1, Acc[1].Tjpa, Vel[1].Tjpa, Pos[1].Tjpa, Dir.el[0][1]);
  queueAddDetails(&(SDI_F->Seg[1]), 3, TNE.Tjna, jerk[1].J1, Acc[1].Taca, Vel[1].Taca, Pos[1].Taca, Dir.el[0][1]);
  queueAddDetails(&(SDI_F->Seg[1]), 4, TNE.Tvc,  jerk[1].J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna, Dir.el[0][1]);
  queueAddDetails(&(SDI_F->Seg[2]), 1, TNE.Tjpa, jerk[2].J1, 0, 0, PFrom.el[0][2], Dir.el[0][2]);
  queueAddDetails(&(SDI_F->Seg[2]), 2, TNE.Taca, jerk[2].J1, Acc[2].Tjpa, Vel[2].Tjpa, Pos[2].Tjpa, Dir.el[0][2]);
  queueAddDetails(&(SDI_F->Seg[2]), 3, TNE.Tjna, jerk[2].J1, Acc[2].Taca, Vel[2].Taca, Pos[2].Taca, Dir.el[0][2]);
  queueAddDetails(&(SDI_F->Seg[2]), 4, TNE.Tvc,  jerk[2].J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna, Dir.el[0][2]);
  queueAddDetails(&(SDI_F->Seg[3]), 1, TNE.Tjpa, jerk[3].J1, 0, 0, QFrom.el[0][0], Dir.el[0][3]);
  queueAddDetails(&(SDI_F->Seg[3]), 2, TNE.Taca, jerk[3].J1, Acc[3].Tjpa, Vel[3].Tjpa, Pos[3].Tjpa, Dir.el[0][3]);
  queueAddDetails(&(SDI_F->Seg[3]), 3, TNE.Tjna, jerk[3].J1, Acc[3].Taca, Vel[3].Taca, Pos[3].Taca, Dir.el[0][3]);
  queueAddDetails(&(SDI_F->Seg[3]), 4, TNE.Tvc,  jerk[3].J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna, Dir.el[0][3]);
  queueAddDetails(&(SDI_F->Seg[4]), 1, TNE.Tjpa, jerk[4].J1, 0, 0, QFrom.el[0][1], Dir.el[0][4]);
  queueAddDetails(&(SDI_F->Seg[4]), 2, TNE.Taca, jerk[4].J1, Acc[4].Tjpa, Vel[4].Tjpa, Pos[4].Tjpa, Dir.el[0][4]);
  queueAddDetails(&(SDI_F->Seg[4]), 3, TNE.Tjna, jerk[4].J1, Acc[4].Taca, Vel[4].Taca, Pos[4].Taca, Dir.el[0][4]);
  queueAddDetails(&(SDI_F->Seg[4]), 4, TNE.Tvc,  jerk[4].J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna, Dir.el[0][4]);
  queueAddDetails(&(SDI_F->Seg[5]), 1, TNE.Tjpa, jerk[5].J1, 0, 0, QFrom.el[0][2], Dir.el[0][5]);
  queueAddDetails(&(SDI_F->Seg[5]), 2, TNE.Taca, jerk[5].J1, Acc[5].Tjpa, Vel[5].Tjpa, Pos[5].Tjpa, Dir.el[0][5]);
  queueAddDetails(&(SDI_F->Seg[5]), 3, TNE.Tjna, jerk[5].J1, Acc[5].Taca, Vel[5].Taca, Pos[5].Taca, Dir.el[0][5]);
  queueAddDetails(&(SDI_F->Seg[5]), 4, TNE.Tvc,  jerk[5].J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna, Dir.el[0][5]);
  queueAddDetails(&(SDI_F->Seg[6]), 1, TNE.Tjpa, jerk[6].J1, 0, 0, QFrom.el[0][3], Dir.el[0][6]);
  queueAddDetails(&(SDI_F->Seg[6]), 2, TNE.Taca, jerk[6].J1, Acc[6].Tjpa, Vel[6].Tjpa, Pos[6].Tjpa, Dir.el[0][6]);
  queueAddDetails(&(SDI_F->Seg[6]), 3, TNE.Tjna, jerk[6].J1, Acc[6].Taca, Vel[6].Taca, Pos[6].Taca, Dir.el[0][6]);
  queueAddDetails(&(SDI_F->Seg[6]), 4, TNE.Tvc,  jerk[6].J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna, Dir.el[0][6]);

  timeSX  = TNE.Tjpa + TNE.Taca + TNE.Tjna + TNE.Tvc;
  timeSY  = timeSX; 
  timeSZ  = timeSX;  
  timeSQN = timeSX;  
  timeSQI = timeSX;  
  timeSQJ = timeSX;  
  timeSQK = timeSX;  

  //  queuePrint(&(SDI_F->Seg[0]));
  //  queuePrint(&(SDI_F->Seg[1]));
  //  queuePrint(&(SDI_F->Seg[2]));
  //  queuePrint(&(SDI_F->Seg[3]));
  //  queuePrint(&(SDI_F->Seg[4]));
  //  queuePrint(&(SDI_F->Seg[5]));
  //  queuePrint(&(SDI_F->Seg[6]));

  if ( SDI_F->consignType == XARM_EXTERN_CONSIGN_TYPE && matPoseTI.rows>1  ) {
    indexTrajectory = 1;
	if (SDI_F->printData == 1) {
	  printf("PX bkp start\n");
	}
    for(i=0;i<=XARM_NB_DIM-1;i++) {
      INITIAL_COND[i].a =  Acc[i].Tvc;
      INITIAL_COND[i].v =  Vel[i].Tvc;  
      INITIAL_COND[i].x =  Pos[i].Tvc; 
    } 
    //   printf("timeSX %d \n",timeSX);
    //   printf("timeSY %d \n",timeSY);
    //   printf("timeSZ %d \n",timeSZ);
    //   printf("timeSQN %d \n",timeSQN);
    //   printf("timeSQI %d \n",timeSQI);
    //   printf("timeSQJ %d \n",timeSQJ);
    //   printf("timeSQK %d \n",timeSQK);
    printf("XARM TRACKING TRAJECTORY MULTI POSE\n");  
    return EXEC;

  } else {
	if (SDI_F->printData == 1) {
	  printf("PX bkp endStart\n");
	}
    /* We send segments to stop the trajectory */
    queueAddDetails(&(SDI_F->Seg[0] ), 5, TNE.Tjnb, jerk[0].J1, Acc[0].Tvc , Vel[0].Tvc , Pos[0].Tvc , Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[0] ), 6, TNE.Tacb, jerk[0].J1, Acc[0].Tjnb, Vel[0].Tjnb, Pos[0].Tjnb, Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[0] ), 7, TNE.Tjpb, jerk[0].J1, Acc[0].Tacb, Vel[0].Tacb, Pos[0].Tacb, Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[1] ), 5, TNE.Tjnb, jerk[1].J1, Acc[1].Tvc , Vel[1].Tvc , Pos[1].Tvc , Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[1] ), 6, TNE.Tacb, jerk[1].J1, Acc[1].Tjnb, Vel[1].Tjnb, Pos[1].Tjnb, Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[1] ), 7, TNE.Tjpb, jerk[1].J1, Acc[1].Tacb, Vel[1].Tacb, Pos[1].Tacb, Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[2] ), 5, TNE.Tjnb, jerk[2].J1, Acc[2].Tvc , Vel[2].Tvc , Pos[2].Tvc , Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[2] ), 6, TNE.Tacb, jerk[2].J1, Acc[2].Tjnb, Vel[2].Tjnb, Pos[2].Tjnb, Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[2] ), 7, TNE.Tjpb, jerk[2].J1, Acc[2].Tacb, Vel[2].Tacb, Pos[2].Tacb, Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[3]), 5, TNE.Tjnb, jerk[3].J1, Acc[3].Tvc , Vel[3].Tvc , Pos[3].Tvc , Dir.el[0][3]);
    queueAddDetails(&(SDI_F->Seg[3]), 6, TNE.Tacb, jerk[3].J1, Acc[3].Tjnb, Vel[3].Tjnb, Pos[3].Tjnb, Dir.el[0][3]);
    queueAddDetails(&(SDI_F->Seg[3]), 7, TNE.Tjpb, jerk[3].J1, Acc[3].Tacb, Vel[3].Tacb, Pos[3].Tacb, Dir.el[0][3]);
    queueAddDetails(&(SDI_F->Seg[4]), 5, TNE.Tjnb, jerk[4].J1, Acc[4].Tvc , Vel[4].Tvc , Pos[4].Tvc , Dir.el[0][4]);
    queueAddDetails(&(SDI_F->Seg[4]), 6, TNE.Tacb, jerk[4].J1, Acc[4].Tjnb, Vel[4].Tjnb, Pos[4].Tjnb, Dir.el[0][4]);
    queueAddDetails(&(SDI_F->Seg[4]), 7, TNE.Tjpb, jerk[4].J1, Acc[4].Tacb, Vel[4].Tacb, Pos[4].Tacb, Dir.el[0][4]);
    queueAddDetails(&(SDI_F->Seg[5]), 5, TNE.Tjnb, jerk[5].J1, Acc[5].Tvc , Vel[5].Tvc , Pos[5].Tvc , Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[5]), 6, TNE.Tacb, jerk[5].J1, Acc[5].Tjnb, Vel[5].Tjnb, Pos[5].Tjnb, Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[5]), 7, TNE.Tjpb, jerk[5].J1, Acc[5].Tacb, Vel[5].Tacb, Pos[5].Tacb, Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[6]), 5, TNE.Tjnb, jerk[6].J1, Acc[6].Tvc , Vel[6].Tvc , Pos[6].Tvc , Dir.el[0][6]);
    queueAddDetails(&(SDI_F->Seg[6]), 6, TNE.Tacb, jerk[6].J1, Acc[6].Tjnb, Vel[6].Tjnb, Pos[6].Tjnb, Dir.el[0][6]);
    queueAddDetails(&(SDI_F->Seg[6]), 7, TNE.Tjpb, jerk[6].J1, Acc[6].Tacb, Vel[6].Tacb, Pos[6].Tacb, Dir.el[0][6]);    
    printf("XARM TRACKING TRAJECTORY SINGLE POSE\n");
    SDI_F->xarmTrackSegment = XARM_TRACK_ON;
    return END; 
  }
}

/* xarmPlanningTrajectoryMain  -  codel EXEC of PlanningTrajectory
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmPlanningTrajectoryMain(int *report)
{
  double cosTheta;
  int i;
  double MaxLDist;                          /* Maximum Linear Distance */
  double MaxADist;                          /* Maximum Angular Distance */
  double MaxDist;                           /* Maximum Distance */
  static double PosStop[XARM_NB_DIM];
  int NOE;  
  TP_JERKS Jerks;
  TP_TIMES  TM, TNE, TM_End, TM_Start, TNE_End, TNE_Start;
  TP_COND IC, FC;
  TP_TIMES Acc[XARM_NB_DIM], Vel[XARM_NB_DIM], Pos[XARM_NB_DIM];
  TP_TIMES Acc_T[XARM_NB_DIM], Vel_T[XARM_NB_DIM], Pos_T[XARM_NB_DIM];
  TP_TIMES Acc_End[XARM_NB_DIM], Vel_End[XARM_NB_DIM], Pos_End[XARM_NB_DIM];
  TP_TIMES Acc_Start[XARM_NB_DIM], Vel_Start[XARM_NB_DIM], Pos_Start[XARM_NB_DIM];
 
  TP_JERKS jerk[XARM_NB_DIM];
  TP_AXIS_TIMES AxisTimes;
  double MaxTime;
  int MaxTimeAxis;
  double MaxTimeM;
  double J;

  /* For optimal transition with stop */
  double OptimalTransitionTimesWithStop[XARM_NB_DIM];
  TP_AXIS_TIMES AxisTimes_End, AxisTimes_Start; 
  int DirTransition_End[XARM_NB_DIM];
  int DirTransition_Start[XARM_NB_DIM];
  int DirTransition_a[XARM_NB_DIM];
  int DirTransition_b[XARM_NB_DIM];
  TP_TIMES TNE_T[XARM_NB_DIM];
  double Jerk_Transition[XARM_NB_DIM];
  double tmEnd, tmStart;
  double DiffTime;
  int AdjustTimeIsOk;
  int TimeIsAdjusted[XARM_NB_DIM];

  int v=0;

  if (indexTrajectory < matPoseTI.rows) {

    if (SDI_F->printData == 1) {
      printf("===================================\n");
      printf("   Transition point %d\n",indexTrajectory);
      printf("===================================\n");
    }
    
    /*=====================================================*/
    /*                Second Trajectory                    */
    /*=====================================================*/ 
    
    /* Position From = Goal Position 1 */
    PFrom.el[0][0] = matPoseTI.el[indexTrajectory-1][0];
    PFrom.el[0][1] = matPoseTI.el[indexTrajectory-1][1];
    PFrom.el[0][2] = matPoseTI.el[indexTrajectory-1][2];
    /* Quaternion From = Goal Orientation 1 */
    QFrom.el[0][0] = matPoseTI.el[indexTrajectory-1][3];
    QFrom.el[0][1] = matPoseTI.el[indexTrajectory-1][4];
    QFrom.el[0][2] = matPoseTI.el[indexTrajectory-1][5];
    QFrom.el[0][3] = matPoseTI.el[indexTrajectory-1][6];
    /* For optimal transition */
    PosStop[0]= PFrom.el[0][0];
    PosStop[1]= PFrom.el[0][1];
    PosStop[2]= PFrom.el[0][2];
    PosStop[3]= QFrom.el[0][0]; 
    PosStop[4]= QFrom.el[0][1]; 
    PosStop[5]= QFrom.el[0][2];
    PosStop[6]= QFrom.el[0][3];

    /* Position To = Goal Position 1 */
    PTo.el[0][0] = matPoseTI.el[indexTrajectory][0];
    PTo.el[0][1] = matPoseTI.el[indexTrajectory][1];
    PTo.el[0][2] = matPoseTI.el[indexTrajectory][2];
    /* Quaternion To = Goal Orientation 1 */
    QTo.el[0][0] = matPoseTI.el[indexTrajectory][3];
    QTo.el[0][1] = matPoseTI.el[indexTrajectory][4];
    QTo.el[0][2] = matPoseTI.el[indexTrajectory][5];
    QTo.el[0][3] = matPoseTI.el[indexTrajectory][6];

    cosTheta = QFrom.el[0][0]*QTo.el[0][0] + QFrom.el[0][1]*QTo.el[0][1] + QFrom.el[0][2]*QTo.el[0][2] + QFrom.el[0][3]*QTo.el[0][3];
    if (cosTheta<0) {
      QTo.el[0][0] = -QTo.el[0][0];
      QTo.el[0][1] = -QTo.el[0][1];
      QTo.el[0][2] = -QTo.el[0][2];
      QTo.el[0][3] = -QTo.el[0][3];
    }
    if(GeometricalCalculations(PFrom, PTo, QFrom, QTo, &MaxLDist, &MaxADist, Dist, LDist, AbsLDist, ADist, AbsADist, AbsDist, Dir, report)!=OK) {
      PRINT_ERROR("xarm ERROR GeometricalCalculations\n");
      return ETHER;
    }
    CalculPointToPointJerkProfile(PFrom, QFrom, SDI_F->pa10Arm.Limits, MaxLDist, MaxADist, MaxDist, AbsDist, Dir, jerk, Acc, Vel, Pos, &TNE, report);
        
    /*=====================================================*/
    /*              Transition Trajectory                  */
    /*=====================================================*/   
      
    /* Increment poster */
    SDI_F->dq.counter += 1;

    if(CalculTransitionWithStopMotion(PosStop, SDI_F->limitsGoto, INITIAL_COND, Acc, Vel, Pos, OptimalTransitionTimesWithStop, &AxisTimes_End, &AxisTimes_Start, DirTransition_End, DirTransition_Start, Acc_End, Vel_End, Pos_End,  Acc_Start, Vel_Start, Pos_Start, report)!=OK) {
      PRINT_ERROR("xarm ERROR CalculTransitionWithStopMotion\n");
      return ETHER;
    }

    /* Compute optimal transition motion for each axis */      
    if(FindMaxTimeAxis(SDI_F->limitsGoto, INITIAL_COND, Acc, Vel, Pos, &MaxTime, &MaxTimeAxis)!=OK) {
      PRINT_ERROR("xarm ERROR FindMaxTimeAxis\n");
      return ETHER;
    }
    tpSamplingAdjustTime(MaxTime, &MaxTimeM);
    MaxTimeM = MaxTimeM *(S_TD/S_T);


    if (SDI_F->saveFiles == 1) {
      for (v=0;v<XARM_NB_DIM;v++) {
	fprintf(fileptr,"%d %f %f %f %f %f %f\n",v,INITIAL_COND[v].a,INITIAL_COND[v].v, 0.0, Acc[v].Tjna, Vel[v].Tjna, ((Pos[v].Tjna) - (INITIAL_COND[v].x)));
      }
      fflush(fileptr);
      v = 0;
    }

    //=============================================//
    //*   Calcul of Jerks Profile for each axis   *//
    //=============================================//

    /* Init of variable */
    Jerks.sel =1;
    AdjustTimeIsOk = 1;
    TimeIsAdjusted[0] = 0;
    TimeIsAdjusted[1] = 0;
    TimeIsAdjusted[2] = 0;
    TimeIsAdjusted[3] = 0;
    TimeIsAdjusted[4] = 0;
    TimeIsAdjusted[5] = 0;
    TimeIsAdjusted[6] = 0;

    //========================================//
    //                IC PX                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[0] < MaxTime) {
      // we stop the motion
      if (ABS(INITIAL_COND[0].v) <soil.linearVelocity && ABS(Vel[0].Tjna)<soil.linearVelocity && ABS(Pos[0].Tjna - PosStop[0])<soil.linearPosition ) {
	if (SDI_F->printData == 1) {
	  printf("PX bkp 1\n");
	}
	queueAddDetails(&(SDI_F->Seg[0]), 4, MaxTimeM,  J, 0, 0, PosStop[0], DirTransition_End[0]); 
	timeSX = timeSX + MaxTimeM;
      }
      else {
	if (SDI_F->printData == 1) {
	  printf("PX bkp 2\n");
	}
	if (tpGetMonotonicTimes(AxisTimes_End.PX, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.PX, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.linear.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[0]), 1, TNE_End.Tjpa, J, INITIAL_COND[0].a, INITIAL_COND[0].v,  INITIAL_COND[0].x, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 2, TNE_End.Taca, J, Acc_End[0].Tjpa, Vel_End[0].Tjpa, Pos_End[0].Tjpa, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 3, TNE_End.Tjna, J, Acc_End[0].Taca, Vel_End[0].Taca, Pos_End[0].Taca, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 4, TNE_End.Tvc,  J, Acc_End[0].Tjna, Vel_End[0].Tjna, Pos_End[0].Tjna, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 5, TNE_End.Tjnb, J, Acc_End[0].Tvc,  Vel_End[0].Tvc,  Pos_End[0].Tvc,  DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 6, TNE_End.Tacb, J, Acc_End[0].Tjnb, Vel_End[0].Tjnb, Pos_End[0].Tjnb, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 7, TNE_End.Tjpb, J, Acc_End[0].Tacb, Vel_End[0].Tacb, Pos_End[0].Tacb, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 4, DiffTime,  J, 0, 0, PosStop[0], DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[0], DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 2, TNE_Start.Taca, J, Acc_Start[0].Tjpa, Vel_Start[0].Tjpa, Pos_Start[0].Tjpa,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 3, TNE_Start.Tjna, J, Acc_Start[0].Taca, Vel_Start[0].Taca, Pos_Start[0].Taca,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 4, TNE_Start.Tvc,  J, Acc_Start[0].Tjna, Vel_Start[0].Tjna, Pos_Start[0].Tjna,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 5, TNE_Start.Tjnb, J, Acc_Start[0].Tvc,  Vel_Start[0].Tvc,  Pos_Start[0].Tvc,   DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 6, TNE_Start.Tacb, J, Acc_Start[0].Tjnb, Vel_Start[0].Tjnb, Pos_Start[0].Tjnb,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 7, TNE_Start.Tjpb, J, Acc_Start[0].Tacb, Vel_Start[0].Tacb, Pos_Start[0].Tacb,  DirTransition_Start[0]);	
	timeSX = timeSX + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;   
      }
    }
    else { 
	if (SDI_F->printData == 1) {
	  printf("PX bkp 3\n");
	}
      //IC.a and FC.a must be zero
      IC.a = INITIAL_COND[0].a;
      IC.v = INITIAL_COND[0].v;
      IC.x = 0;
      FC.a = Acc[0].Tjna;
      FC.v = Vel[0].Tjna;
      FC.x = (Pos[0].Tjna) - (INITIAL_COND[0].x);
 
      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.linear, &(AxisTimes.PX), &Jerk_Transition[0], &DirTransition_a[0], &DirTransition_b[0])!=0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;
	if (SDI_F->printData == 1) {
	  printf("PX bkp 4\n");
	}
	if (tpGetMonotonicTimes(AxisTimes_End.PX, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.PX, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);
	
	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.linear.maxJerk;
	
	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[0]), 1, TNE_End.Tjpa, J, INITIAL_COND[0].a, INITIAL_COND[0].v,  INITIAL_COND[0].x, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 2, TNE_End.Taca, J, Acc_End[0].Tjpa, Vel_End[0].Tjpa, Pos_End[0].Tjpa, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 3, TNE_End.Tjna, J, Acc_End[0].Taca, Vel_End[0].Taca, Pos_End[0].Taca, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 4, TNE_End.Tvc,  J, Acc_End[0].Tjna, Vel_End[0].Tjna, Pos_End[0].Tjna, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 5, TNE_End.Tjnb, J, Acc_End[0].Tvc,  Vel_End[0].Tvc,  Pos_End[0].Tvc,  DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 6, TNE_End.Tacb, J, Acc_End[0].Tjnb, Vel_End[0].Tjnb, Pos_End[0].Tjnb, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 7, TNE_End.Tjpb, J, Acc_End[0].Tacb, Vel_End[0].Tacb, Pos_End[0].Tacb, DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 4, DiffTime,  J, 0, 0, PosStop[0], DirTransition_End[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[0], DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 2, TNE_Start.Taca, J, Acc_Start[0].Tjpa, Vel_Start[0].Tjpa, Pos_Start[0].Tjpa,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 3, TNE_Start.Tjna, J, Acc_Start[0].Taca, Vel_Start[0].Taca, Pos_Start[0].Taca,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 4, TNE_Start.Tvc,  J, Acc_Start[0].Tjna, Vel_Start[0].Tjna, Pos_Start[0].Tjna,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 5, TNE_Start.Tjnb, J, Acc_Start[0].Tvc,  Vel_Start[0].Tvc,  Pos_Start[0].Tvc,   DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 6, TNE_Start.Tacb, J, Acc_Start[0].Tjnb, Vel_Start[0].Tjnb, Pos_Start[0].Tjnb,  DirTransition_Start[0]);
	queueAddDetails(&(SDI_F->Seg[0]), 7, TNE_Start.Tjpb, J, Acc_Start[0].Tacb, Vel_Start[0].Tacb, Pos_Start[0].Tacb,  DirTransition_Start[0]);	
	timeSX = timeSX + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;

     
      } else {
	if (SDI_F->printData == 1) {
	  printf("PX bkp 5\n");
	}
	/* Verify Times */
	IC.x = INITIAL_COND[0].x ;
	FC.x = (Pos[0].Tjna) - (INITIAL_COND[0].x);
	Jerks.J1 = Jerk_Transition[0];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_LINEAR, FC.x, Jerks, IC, DirTransition_a[0], DirTransition_b[0], (AxisTimes.PX) , &FC, &Acc_T[0], &Vel_T[0], &Pos_T[0])!=OK) {
	  PRINT_ERROR(" Verify Times PX");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.PX), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[0]);
	TimeIsAdjusted[0] = 1;
	if (SDI_F->printData == 1) {
	  printf("PX bkp 6\n");
	}
      }
    }

    //========================================//
    //                IC PY                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[1] < MaxTime) {
       if (ABS(INITIAL_COND[1].v) < soil.linearVelocity && ABS(Vel[1].Tjna)<soil.linearVelocity && ABS(Pos[1].Tjna - PosStop[1])<soil.linearPosition ) {
 	queueAddDetails(&(SDI_F->Seg[1]), 4, MaxTimeM,  J, 0, 0, PosStop[1], DirTransition_End[1]); 
 	timeSY = timeSY + MaxTimeM;
       }
       else {
	if (tpGetMonotonicTimes(AxisTimes_End.PY, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.PY, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.linear.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[1]), 1, TNE_End.Tjpa, J, INITIAL_COND[1].a, INITIAL_COND[1].v,  INITIAL_COND[1].x, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 2, TNE_End.Taca, J, Acc_End[1].Tjpa, Vel_End[1].Tjpa, Pos_End[1].Tjpa, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 3, TNE_End.Tjna, J, Acc_End[1].Taca, Vel_End[1].Taca, Pos_End[1].Taca, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 4, TNE_End.Tvc,  J, Acc_End[1].Tjna, Vel_End[1].Tjna, Pos_End[1].Tjna, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 5, TNE_End.Tjnb, J, Acc_End[1].Tvc,  Vel_End[1].Tvc,  Pos_End[1].Tvc,  DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 6, TNE_End.Tacb, J, Acc_End[1].Tjnb, Vel_End[1].Tjnb, Pos_End[1].Tjnb, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 7, TNE_End.Tjpb, J, Acc_End[1].Tacb, Vel_End[1].Tacb, Pos_End[1].Tacb, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 4, DiffTime,  J, 0, 0, PosStop[1], DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[1], DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 2, TNE_Start.Taca, J, Acc_Start[1].Tjpa, Vel_Start[1].Tjpa, Pos_Start[1].Tjpa,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 3, TNE_Start.Tjna, J, Acc_Start[1].Taca, Vel_Start[1].Taca, Pos_Start[1].Taca,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 4, TNE_Start.Tvc,  J, Acc_Start[1].Tjna, Vel_Start[1].Tjna, Pos_Start[1].Tjna,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 5, TNE_Start.Tjnb, J, Acc_Start[1].Tvc,  Vel_Start[1].Tvc,  Pos_Start[1].Tvc,   DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 6, TNE_Start.Tacb, J, Acc_Start[1].Tjnb, Vel_Start[1].Tjnb, Pos_Start[1].Tjnb,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 7, TNE_Start.Tjpb, J, Acc_Start[1].Tacb, Vel_Start[1].Tacb, Pos_Start[1].Tacb,  DirTransition_Start[1]);		
	timeSY = timeSY + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       }
    }
    else { 

      IC.a = INITIAL_COND[1].a;
      IC.v = INITIAL_COND[1].v;
      IC.x = 0;
      FC.a = Acc[1].Tjna;
      FC.v = Vel[1].Tjna;
      FC.x = (Pos[1].Tjna) - (INITIAL_COND[1].x);

      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.linear, &(AxisTimes.PY), &Jerk_Transition[1], &DirTransition_a[1], &DirTransition_b[1])!=0 || AdjustTimeIsOk == 0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;

	if (tpGetMonotonicTimes(AxisTimes_End.PY, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.PY, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.linear.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[1]), 1, TNE_End.Tjpa, J, INITIAL_COND[1].a, INITIAL_COND[1].v,  INITIAL_COND[1].x, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 2, TNE_End.Taca, J, Acc_End[1].Tjpa, Vel_End[1].Tjpa, Pos_End[1].Tjpa, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 3, TNE_End.Tjna, J, Acc_End[1].Taca, Vel_End[1].Taca, Pos_End[1].Taca, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 4, TNE_End.Tvc,  J, Acc_End[1].Tjna, Vel_End[1].Tjna, Pos_End[1].Tjna, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 5, TNE_End.Tjnb, J, Acc_End[1].Tvc,  Vel_End[1].Tvc,  Pos_End[1].Tvc,  DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 6, TNE_End.Tacb, J, Acc_End[1].Tjnb, Vel_End[1].Tjnb, Pos_End[1].Tjnb, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 7, TNE_End.Tjpb, J, Acc_End[1].Tacb, Vel_End[1].Tacb, Pos_End[1].Tacb, DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 4, DiffTime,  J, 0, 0, PosStop[1], DirTransition_End[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[1], DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 2, TNE_Start.Taca, J, Acc_Start[1].Tjpa, Vel_Start[1].Tjpa, Pos_Start[1].Tjpa,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 3, TNE_Start.Tjna, J, Acc_Start[1].Taca, Vel_Start[1].Taca, Pos_Start[1].Taca,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 4, TNE_Start.Tvc,  J, Acc_Start[1].Tjna, Vel_Start[1].Tjna, Pos_Start[1].Tjna,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 5, TNE_Start.Tjnb, J, Acc_Start[1].Tvc,  Vel_Start[1].Tvc,  Pos_Start[1].Tvc,   DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 6, TNE_Start.Tacb, J, Acc_Start[1].Tjnb, Vel_Start[1].Tjnb, Pos_Start[1].Tjnb,  DirTransition_Start[1]);
	queueAddDetails(&(SDI_F->Seg[1]), 7, TNE_Start.Tjpb, J, Acc_Start[1].Tacb, Vel_Start[1].Tacb, Pos_Start[1].Tacb,  DirTransition_Start[1]);		
	timeSY = timeSY + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
     
      } else {
	IC.x = INITIAL_COND[1].x ;
	FC.x = (Pos[1].Tjna) - (INITIAL_COND[1].x);
	Jerks.J1 = Jerk_Transition[1];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_LINEAR, FC.x, Jerks, IC, DirTransition_a[1], DirTransition_b[1], (AxisTimes.PY) , &FC, &Acc_T[1], &Vel_T[1], &Pos_T[1])!=OK) {
	  PRINT_ERROR(" Verify Times PY");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.PY), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[1]);
	TimeIsAdjusted[1] = 1;
      }
    }

    //========================================//
    //                IC PZ                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[2] < MaxTime) {
         if (ABS(INITIAL_COND[2].v) < soil.linearVelocity && ABS(Vel[2].Tjna)<soil.linearVelocity && ABS(Pos[2].Tjna - PosStop[2])<soil.linearPosition ) {
 	queueAddDetails(&(SDI_F->Seg[2]), 4, MaxTimeM,  J, 0, 0, PosStop[2], DirTransition_End[2]); 
 	timeSZ = timeSZ + MaxTimeM;
       }
       else {
	if (tpGetMonotonicTimes(AxisTimes_End.PZ, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.PZ, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.linear.maxJerk;
	
	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[2]), 1, TNE_End.Tjpa, J, INITIAL_COND[2].a, INITIAL_COND[2].v,  INITIAL_COND[2].x, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 2, TNE_End.Taca, J, Acc_End[2].Tjpa, Vel_End[2].Tjpa, Pos_End[2].Tjpa, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 3, TNE_End.Tjna, J, Acc_End[2].Taca, Vel_End[2].Taca, Pos_End[2].Taca, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 4, TNE_End.Tvc,  J, Acc_End[2].Tjna, Vel_End[2].Tjna, Pos_End[2].Tjna, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 5, TNE_End.Tjnb, J, Acc_End[2].Tvc,  Vel_End[2].Tvc,  Pos_End[2].Tvc,  DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 6, TNE_End.Tacb, J, Acc_End[2].Tjnb, Vel_End[2].Tjnb, Pos_End[2].Tjnb, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 7, TNE_End.Tjpb, J, Acc_End[2].Tacb, Vel_End[2].Tacb, Pos_End[2].Tacb, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 4, DiffTime,  J, 0, 0, PosStop[2], DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[2], DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 2, TNE_Start.Taca, J, Acc_Start[2].Tjpa, Vel_Start[2].Tjpa, Pos_Start[2].Tjpa,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 3, TNE_Start.Tjna, J, Acc_Start[2].Taca, Vel_Start[2].Taca, Pos_Start[2].Taca,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 4, TNE_Start.Tvc,  J, Acc_Start[2].Tjna, Vel_Start[2].Tjna, Pos_Start[2].Tjna,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 5, TNE_Start.Tjnb, J, Acc_Start[2].Tvc,  Vel_Start[2].Tvc,  Pos_Start[2].Tvc,   DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 6, TNE_Start.Tacb, J, Acc_Start[2].Tjnb, Vel_Start[2].Tjnb, Pos_Start[2].Tjnb,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 7, TNE_Start.Tjpb, J, Acc_Start[2].Tacb, Vel_Start[2].Tacb, Pos_Start[2].Tacb,  DirTransition_Start[2]);		
	timeSZ = timeSZ + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       }
    }
    else { 
      IC.a = INITIAL_COND[2].a;
      IC.v = INITIAL_COND[2].v;
      IC.x = 0;
      FC.a = Acc[2].Tjna;
      FC.v = Vel[2].Tjna;
      FC.x = (Pos[2].Tjna) - (INITIAL_COND[2].x);

      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.linear, &(AxisTimes.PZ), &Jerk_Transition[2], &DirTransition_a[2], &DirTransition_b[2])!=0 || AdjustTimeIsOk == 0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;

	if (tpGetMonotonicTimes(AxisTimes_End.PZ, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.PZ, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.linear.maxJerk;
	
	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[2]), 1, TNE_End.Tjpa, J, INITIAL_COND[2].a, INITIAL_COND[2].v,  INITIAL_COND[2].x, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 2, TNE_End.Taca, J, Acc_End[2].Tjpa, Vel_End[2].Tjpa, Pos_End[2].Tjpa, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 3, TNE_End.Tjna, J, Acc_End[2].Taca, Vel_End[2].Taca, Pos_End[2].Taca, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 4, TNE_End.Tvc,  J, Acc_End[2].Tjna, Vel_End[2].Tjna, Pos_End[2].Tjna, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 5, TNE_End.Tjnb, J, Acc_End[2].Tvc,  Vel_End[2].Tvc,  Pos_End[2].Tvc,  DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 6, TNE_End.Tacb, J, Acc_End[2].Tjnb, Vel_End[2].Tjnb, Pos_End[2].Tjnb, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 7, TNE_End.Tjpb, J, Acc_End[2].Tacb, Vel_End[2].Tacb, Pos_End[2].Tacb, DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 4, DiffTime,  J, 0, 0, PosStop[2], DirTransition_End[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[2], DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 2, TNE_Start.Taca, J, Acc_Start[2].Tjpa, Vel_Start[2].Tjpa, Pos_Start[2].Tjpa,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 3, TNE_Start.Tjna, J, Acc_Start[2].Taca, Vel_Start[2].Taca, Pos_Start[2].Taca,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 4, TNE_Start.Tvc,  J, Acc_Start[2].Tjna, Vel_Start[2].Tjna, Pos_Start[2].Tjna,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 5, TNE_Start.Tjnb, J, Acc_Start[2].Tvc,  Vel_Start[2].Tvc,  Pos_Start[2].Tvc,   DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 6, TNE_Start.Tacb, J, Acc_Start[2].Tjnb, Vel_Start[2].Tjnb, Pos_Start[2].Tjnb,  DirTransition_Start[2]);
	queueAddDetails(&(SDI_F->Seg[2]), 7, TNE_Start.Tjpb, J, Acc_Start[2].Tacb, Vel_Start[2].Tacb, Pos_Start[2].Tacb,  DirTransition_Start[2]);		
	timeSZ = timeSZ + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;

      } else {
	 IC.a = INITIAL_COND[2].a;
	 IC.v = INITIAL_COND[2].v;
	 IC.x = INITIAL_COND[2].x;
	 FC.a = Acc[2].Tjna;
	 FC.v = Vel[2].Tjna;
	 FC.x = (Pos[2].Tjna) - (INITIAL_COND[2].x);

	Jerks.J1 = Jerk_Transition[2];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_LINEAR, FC.x, Jerks, IC, DirTransition_a[2], DirTransition_b[2], (AxisTimes.PZ) , &FC, &Acc_T[2], &Vel_T[2], &Pos_T[2])!=OK) {
	  PRINT_ERROR(" Verify Times PZ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.PZ), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[2]);
	TimeIsAdjusted[2] = 1;
      }
    }

    //========================================//
    //                IC QN                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[3] < MaxTime) {
       if (ABS(INITIAL_COND[3].v) < soil.angularVelocity && ABS(Vel[3].Tjna)<soil.angularVelocity && ABS(Pos[3].Tjna - PosStop[3])<soil.angularPosition ) {
 	queueAddDetails(&(SDI_F->Seg[3]), 4, MaxTimeM,  J, 0, 0, PosStop[3], DirTransition_End[3]); 
 	timeSQN = timeSQN + MaxTimeM;
       }
       else {
	if (tpGetMonotonicTimes(AxisTimes_End.QN, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QN, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[3]), 1, TNE_End.Tjpa, J, INITIAL_COND[3].a, INITIAL_COND[3].v,  INITIAL_COND[3].x, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 2, TNE_End.Taca, J, Acc_End[3].Tjpa, Vel_End[3].Tjpa, Pos_End[3].Tjpa, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 3, TNE_End.Tjna, J, Acc_End[3].Taca, Vel_End[3].Taca, Pos_End[3].Taca, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 4, TNE_End.Tvc,  J, Acc_End[3].Tjna, Vel_End[3].Tjna, Pos_End[3].Tjna, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 5, TNE_End.Tjnb, J, Acc_End[3].Tvc,  Vel_End[3].Tvc,  Pos_End[3].Tvc,  DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 6, TNE_End.Tacb, J, Acc_End[3].Tjnb, Vel_End[3].Tjnb, Pos_End[3].Tjnb, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 7, TNE_End.Tjpb, J, Acc_End[3].Tacb, Vel_End[3].Tacb, Pos_End[3].Tacb, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 4, DiffTime,  J, 0, 0, PosStop[3], DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[3], DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 2, TNE_Start.Taca, J, Acc_Start[3].Tjpa, Vel_Start[3].Tjpa, Pos_Start[3].Tjpa,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 3, TNE_Start.Tjna, J, Acc_Start[3].Taca, Vel_Start[3].Taca, Pos_Start[3].Taca,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 4, TNE_Start.Tvc,  J, Acc_Start[3].Tjna, Vel_Start[3].Tjna, Pos_Start[3].Tjna,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 5, TNE_Start.Tjnb, J, Acc_Start[3].Tvc,  Vel_Start[3].Tvc,  Pos_Start[3].Tvc,   DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 6, TNE_Start.Tacb, J, Acc_Start[3].Tjnb, Vel_Start[3].Tjnb, Pos_Start[3].Tjnb,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 7, TNE_Start.Tjpb, J, Acc_Start[3].Tacb, Vel_Start[3].Tacb, Pos_Start[3].Tacb,  DirTransition_Start[3]);		
	timeSQN = timeSQN + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       }
    }
    else { 
      IC.a = INITIAL_COND[3].a;
      IC.v = INITIAL_COND[3].v;
      IC.x = 0;
      FC.a = Acc[3].Tjna;
      FC.v = Vel[3].Tjna;
      FC.x = (Pos[3].Tjna) - (INITIAL_COND[3].x);

      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.angular, &(AxisTimes.QN), &Jerk_Transition[3], &DirTransition_a[3], &DirTransition_b[3])!=0 || AdjustTimeIsOk == 0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;

	if (tpGetMonotonicTimes(AxisTimes_End.QN, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QN, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[3]), 1, TNE_End.Tjpa, J, INITIAL_COND[3].a, INITIAL_COND[3].v,  INITIAL_COND[3].x, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 2, TNE_End.Taca, J, Acc_End[3].Tjpa, Vel_End[3].Tjpa, Pos_End[3].Tjpa, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 3, TNE_End.Tjna, J, Acc_End[3].Taca, Vel_End[3].Taca, Pos_End[3].Taca, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 4, TNE_End.Tvc,  J, Acc_End[3].Tjna, Vel_End[3].Tjna, Pos_End[3].Tjna, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 5, TNE_End.Tjnb, J, Acc_End[3].Tvc,  Vel_End[3].Tvc,  Pos_End[3].Tvc,  DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 6, TNE_End.Tacb, J, Acc_End[3].Tjnb, Vel_End[3].Tjnb, Pos_End[3].Tjnb, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 7, TNE_End.Tjpb, J, Acc_End[3].Tacb, Vel_End[3].Tacb, Pos_End[3].Tacb, DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 4, DiffTime,  J, 0, 0, PosStop[3], DirTransition_End[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[3], DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 2, TNE_Start.Taca, J, Acc_Start[3].Tjpa, Vel_Start[3].Tjpa, Pos_Start[3].Tjpa,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 3, TNE_Start.Tjna, J, Acc_Start[3].Taca, Vel_Start[3].Taca, Pos_Start[3].Taca,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 4, TNE_Start.Tvc,  J, Acc_Start[3].Tjna, Vel_Start[3].Tjna, Pos_Start[3].Tjna,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 5, TNE_Start.Tjnb, J, Acc_Start[3].Tvc,  Vel_Start[3].Tvc,  Pos_Start[3].Tvc,   DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 6, TNE_Start.Tacb, J, Acc_Start[3].Tjnb, Vel_Start[3].Tjnb, Pos_Start[3].Tjnb,  DirTransition_Start[3]);
	queueAddDetails(&(SDI_F->Seg[3]), 7, TNE_Start.Tjpb, J, Acc_Start[3].Tacb, Vel_Start[3].Tacb, Pos_Start[3].Tacb,  DirTransition_Start[3]);		
	timeSQN = timeSQN + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
     
      } else {
	IC.x = INITIAL_COND[3].x ;
	FC.x = (Pos[3].Tjna) - (INITIAL_COND[3].x);
	Jerks.J1 = Jerk_Transition[3];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_ANGULAR, FC.x, Jerks, IC, DirTransition_a[3], DirTransition_b[3], (AxisTimes.QN) , &FC, &Acc_T[3], &Vel_T[3], &Pos_T[3])!=OK) {
	  PRINT_ERROR(" Verify Times QN");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.QN), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[3]);
	TimeIsAdjusted[3] = 1;
      }
    }

    //========================================//
    //                IC QI                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[4] < MaxTime) {
       if (ABS(INITIAL_COND[4].v) < soil.angularVelocity && ABS(Vel[4].Tjna)<soil.angularVelocity && ABS(Pos[4].Tjna - PosStop[4])<soil.angularPosition ) {
 	queueAddDetails(&(SDI_F->Seg[4]), 4, MaxTimeM,  J, 0, 0, PosStop[4], DirTransition_End[4]); 
 	timeSQI = timeSQI + MaxTimeM;
       }
       else {
	if (tpGetMonotonicTimes(AxisTimes_End.QI, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QI, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[4]), 1, TNE_End.Tjpa, J, INITIAL_COND[4].a, INITIAL_COND[4].v,  INITIAL_COND[4].x, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 2, TNE_End.Taca, J, Acc_End[4].Tjpa, Vel_End[4].Tjpa, Pos_End[4].Tjpa, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 3, TNE_End.Tjna, J, Acc_End[4].Taca, Vel_End[4].Taca, Pos_End[4].Taca, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 4, TNE_End.Tvc,  J, Acc_End[4].Tjna, Vel_End[4].Tjna, Pos_End[4].Tjna, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 5, TNE_End.Tjnb, J, Acc_End[4].Tvc,  Vel_End[4].Tvc,  Pos_End[4].Tvc,  DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 6, TNE_End.Tacb, J, Acc_End[4].Tjnb, Vel_End[4].Tjnb, Pos_End[4].Tjnb, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 7, TNE_End.Tjpb, J, Acc_End[4].Tacb, Vel_End[4].Tacb, Pos_End[4].Tacb, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 4, DiffTime,  J, 0, 0, PosStop[4], DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[4], DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 2, TNE_Start.Taca, J, Acc_Start[4].Tjpa, Vel_Start[4].Tjpa, Pos_Start[4].Tjpa,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 3, TNE_Start.Tjna, J, Acc_Start[4].Taca, Vel_Start[4].Taca, Pos_Start[4].Taca,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 4, TNE_Start.Tvc,  J, Acc_Start[4].Tjna, Vel_Start[4].Tjna, Pos_Start[4].Tjna,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 5, TNE_Start.Tjnb, J, Acc_Start[4].Tvc,  Vel_Start[4].Tvc,  Pos_Start[4].Tvc,   DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 6, TNE_Start.Tacb, J, Acc_Start[4].Tjnb, Vel_Start[4].Tjnb, Pos_Start[4].Tjnb,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 7, TNE_Start.Tjpb, J, Acc_Start[4].Tacb, Vel_Start[4].Tacb, Pos_Start[4].Tacb,  DirTransition_Start[4]);		
	timeSQI = timeSQI + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       }
    }
    else { 
      IC.a = INITIAL_COND[4].a;
      IC.v = INITIAL_COND[4].v;
      IC.x = 0;
      FC.a = Acc[4].Tjna;
      FC.v = Vel[4].Tjna;
      FC.x = (Pos[4].Tjna) - (INITIAL_COND[4].x);

      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.angular, &(AxisTimes.QI), &Jerk_Transition[4], &DirTransition_a[4], &DirTransition_b[4])!=0 || AdjustTimeIsOk == 0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;

	if (tpGetMonotonicTimes(AxisTimes_End.QI, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QI, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[4]), 1, TNE_End.Tjpa, J, INITIAL_COND[4].a, INITIAL_COND[4].v,  INITIAL_COND[4].x, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 2, TNE_End.Taca, J, Acc_End[4].Tjpa, Vel_End[4].Tjpa, Pos_End[4].Tjpa, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 3, TNE_End.Tjna, J, Acc_End[4].Taca, Vel_End[4].Taca, Pos_End[4].Taca, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 4, TNE_End.Tvc,  J, Acc_End[4].Tjna, Vel_End[4].Tjna, Pos_End[4].Tjna, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 5, TNE_End.Tjnb, J, Acc_End[4].Tvc,  Vel_End[4].Tvc,  Pos_End[4].Tvc,  DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 6, TNE_End.Tacb, J, Acc_End[4].Tjnb, Vel_End[4].Tjnb, Pos_End[4].Tjnb, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 7, TNE_End.Tjpb, J, Acc_End[4].Tacb, Vel_End[4].Tacb, Pos_End[4].Tacb, DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 4, DiffTime,  J, 0, 0, PosStop[4], DirTransition_End[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[4], DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 2, TNE_Start.Taca, J, Acc_Start[4].Tjpa, Vel_Start[4].Tjpa, Pos_Start[4].Tjpa,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 3, TNE_Start.Tjna, J, Acc_Start[4].Taca, Vel_Start[4].Taca, Pos_Start[4].Taca,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 4, TNE_Start.Tvc,  J, Acc_Start[4].Tjna, Vel_Start[4].Tjna, Pos_Start[4].Tjna,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 5, TNE_Start.Tjnb, J, Acc_Start[4].Tvc,  Vel_Start[4].Tvc,  Pos_Start[4].Tvc,   DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 6, TNE_Start.Tacb, J, Acc_Start[4].Tjnb, Vel_Start[4].Tjnb, Pos_Start[4].Tjnb,  DirTransition_Start[4]);
	queueAddDetails(&(SDI_F->Seg[4]), 7, TNE_Start.Tjpb, J, Acc_Start[4].Tacb, Vel_Start[4].Tacb, Pos_Start[4].Tacb,  DirTransition_Start[4]);		
	timeSQI = timeSQI + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
      
      } else {
	IC.x = INITIAL_COND[4].x ;
	FC.x = (Pos[4].Tjna) - (INITIAL_COND[4].x);
	Jerks.J1 = Jerk_Transition[4];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_ANGULAR, FC.x, Jerks, IC, DirTransition_a[4], DirTransition_b[4], (AxisTimes.QI) , &FC, &Acc_T[4], &Vel_T[4], &Pos_T[4])!=OK) {
	  PRINT_ERROR(" Verify Times QI");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.QI), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[4]);
	TimeIsAdjusted[4] = 1;
      }
    }

    //========================================//
    //                IC QJ                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[5] < MaxTime) {
       if (ABS(INITIAL_COND[5].v) <soil.angularVelocity && ABS(Vel[5].Tjna)<soil.angularVelocity && ABS(Pos[5].Tjna - PosStop[5])<soil.angularPosition ) {
 	queueAddDetails(&(SDI_F->Seg[5]), 4, MaxTimeM,  J, 0, 0, PosStop[5], DirTransition_End[5]); 
 	timeSQJ = timeSQJ + MaxTimeM;
       }
       else {
	if (tpGetMonotonicTimes(AxisTimes_End.QJ, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QJ, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[5]), 1, TNE_End.Tjpa, J, INITIAL_COND[5].a, INITIAL_COND[5].v,  INITIAL_COND[5].x, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 2, TNE_End.Taca, J, Acc_End[5].Tjpa, Vel_End[5].Tjpa, Pos_End[5].Tjpa, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 3, TNE_End.Tjna, J, Acc_End[5].Taca, Vel_End[5].Taca, Pos_End[5].Taca, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 4, TNE_End.Tvc,  J, Acc_End[5].Tjna, Vel_End[5].Tjna, Pos_End[5].Tjna, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 5, TNE_End.Tjnb, J, Acc_End[5].Tvc,  Vel_End[5].Tvc,  Pos_End[5].Tvc,  DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 6, TNE_End.Tacb, J, Acc_End[5].Tjnb, Vel_End[5].Tjnb, Pos_End[5].Tjnb, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 7, TNE_End.Tjpb, J, Acc_End[5].Tacb, Vel_End[5].Tacb, Pos_End[5].Tacb, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 4, DiffTime,  J, 0, 0, PosStop[5], DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[5], DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 2, TNE_Start.Taca, J, Acc_Start[5].Tjpa, Vel_Start[5].Tjpa, Pos_Start[5].Tjpa,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 3, TNE_Start.Tjna, J, Acc_Start[5].Taca, Vel_Start[5].Taca, Pos_Start[5].Taca,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 4, TNE_Start.Tvc,  J, Acc_Start[5].Tjna, Vel_Start[5].Tjna, Pos_Start[5].Tjna,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 5, TNE_Start.Tjnb, J, Acc_Start[5].Tvc,  Vel_Start[5].Tvc,  Pos_Start[5].Tvc,   DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 6, TNE_Start.Tacb, J, Acc_Start[5].Tjnb, Vel_Start[5].Tjnb, Pos_Start[5].Tjnb,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 7, TNE_Start.Tjpb, J, Acc_Start[5].Tacb, Vel_Start[5].Tacb, Pos_Start[5].Tacb,  DirTransition_Start[5]);		
	timeSQJ = timeSQJ + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       }
    }
    else { 
      IC.a = INITIAL_COND[5].a;
      IC.v = INITIAL_COND[5].v;
      IC.x = 0;
      FC.a = Acc[5].Tjna;
      FC.v = Vel[5].Tjna;
      FC.x = (Pos[5].Tjna) - (INITIAL_COND[5].x);

      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.angular, &(AxisTimes.QJ), &Jerk_Transition[5], &DirTransition_a[5], &DirTransition_b[5])!=0 || AdjustTimeIsOk == 0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;

    	if (tpGetMonotonicTimes(AxisTimes_End.QJ, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QJ, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[5]), 1, TNE_End.Tjpa, J, INITIAL_COND[5].a, INITIAL_COND[5].v,  INITIAL_COND[5].x, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 2, TNE_End.Taca, J, Acc_End[5].Tjpa, Vel_End[5].Tjpa, Pos_End[5].Tjpa, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 3, TNE_End.Tjna, J, Acc_End[5].Taca, Vel_End[5].Taca, Pos_End[5].Taca, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 4, TNE_End.Tvc,  J, Acc_End[5].Tjna, Vel_End[5].Tjna, Pos_End[5].Tjna, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 5, TNE_End.Tjnb, J, Acc_End[5].Tvc,  Vel_End[5].Tvc,  Pos_End[5].Tvc,  DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 6, TNE_End.Tacb, J, Acc_End[5].Tjnb, Vel_End[5].Tjnb, Pos_End[5].Tjnb, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 7, TNE_End.Tjpb, J, Acc_End[5].Tacb, Vel_End[5].Tacb, Pos_End[5].Tacb, DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 4, DiffTime,  J, 0, 0, PosStop[5], DirTransition_End[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[5], DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 2, TNE_Start.Taca, J, Acc_Start[5].Tjpa, Vel_Start[5].Tjpa, Pos_Start[5].Tjpa,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 3, TNE_Start.Tjna, J, Acc_Start[5].Taca, Vel_Start[5].Taca, Pos_Start[5].Taca,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 4, TNE_Start.Tvc,  J, Acc_Start[5].Tjna, Vel_Start[5].Tjna, Pos_Start[5].Tjna,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 5, TNE_Start.Tjnb, J, Acc_Start[5].Tvc,  Vel_Start[5].Tvc,  Pos_Start[5].Tvc,   DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 6, TNE_Start.Tacb, J, Acc_Start[5].Tjnb, Vel_Start[5].Tjnb, Pos_Start[5].Tjnb,  DirTransition_Start[5]);
	queueAddDetails(&(SDI_F->Seg[5]), 7, TNE_Start.Tjpb, J, Acc_Start[5].Tacb, Vel_Start[5].Tacb, Pos_Start[5].Tacb,  DirTransition_Start[5]);		
	timeSQJ = timeSQJ + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
      
      } else {
	IC.x = INITIAL_COND[5].x ;
	FC.x = (Pos[5].Tjna) - (INITIAL_COND[5].x);
	Jerks.J1 = Jerk_Transition[5];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_ANGULAR, FC.x, Jerks, IC, DirTransition_a[5], DirTransition_b[5], (AxisTimes.QJ) , &FC, &Acc_T[5], &Vel_T[5], &Pos_T[5])!=OK) {
	  PRINT_ERROR(" Verify Times QJ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.QJ), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[5]);
	TimeIsAdjusted[5] = 1;
      }
    }

    //========================================//
    //                IC QK                   //
    //========================================//
    if (OptimalTransitionTimesWithStop[6] < MaxTime) {
       if (ABS(INITIAL_COND[6].v) <soil.angularVelocity && ABS(Vel[6].Tjna)<soil.angularVelocity && ABS(Pos[6].Tjna - PosStop[6])<soil.angularPosition ) {
 	queueAddDetails(&(SDI_F->Seg[6]), 4, MaxTimeM,  J, 0, 0, PosStop[6], DirTransition_End[6]); 
 	timeSQK = timeSQK + MaxTimeM;
       }
       else {
	if (tpGetMonotonicTimes(AxisTimes_End.QK, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
 	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QK, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[6]), 1, TNE_End.Tjpa, J, INITIAL_COND[6].a, INITIAL_COND[6].v,  INITIAL_COND[6].x, DirTransition_End[6]);
	
	queueAddDetails(&(SDI_F->Seg[6]), 2, TNE_End.Taca, J, Acc_End[6].Tjpa, Vel_End[6].Tjpa, Pos_End[6].Tjpa, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 3, TNE_End.Tjna, J, Acc_End[6].Taca, Vel_End[6].Taca, Pos_End[6].Taca, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 4, TNE_End.Tvc,  J, Acc_End[6].Tjna, Vel_End[6].Tjna, Pos_End[6].Tjna, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 5, TNE_End.Tjnb, J, Acc_End[6].Tvc,  Vel_End[6].Tvc,  Pos_End[6].Tvc,  DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 6, TNE_End.Tacb, J, Acc_End[6].Tjnb, Vel_End[6].Tjnb, Pos_End[6].Tjnb, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 7, TNE_End.Tjpb, J, Acc_End[6].Tacb, Vel_End[6].Tacb, Pos_End[6].Tacb, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 4, DiffTime,  J, 0, 0, PosStop[6], DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[6], DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 2, TNE_Start.Taca, J, Acc_Start[6].Tjpa, Vel_Start[6].Tjpa, Pos_Start[6].Tjpa,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 3, TNE_Start.Tjna, J, Acc_Start[6].Taca, Vel_Start[6].Taca, Pos_Start[6].Taca,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 4, TNE_Start.Tvc,  J, Acc_Start[6].Tjna, Vel_Start[6].Tjna, Pos_Start[6].Tjna,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 5, TNE_Start.Tjnb, J, Acc_Start[6].Tvc,  Vel_Start[6].Tvc,  Pos_Start[6].Tvc,   DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 6, TNE_Start.Tacb, J, Acc_Start[6].Tjnb, Vel_Start[6].Tjnb, Pos_Start[6].Tjnb,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 7, TNE_Start.Tjpb, J, Acc_Start[6].Tacb, Vel_Start[6].Tacb, Pos_Start[6].Tacb,  DirTransition_Start[6]);
	timeSQK = timeSQK + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       }
    }
    else { 
      IC.a = INITIAL_COND[6].a;
      IC.v = INITIAL_COND[6].v;
      IC.x = 0;
      FC.a = Acc[6].Tjna;
      FC.v = Vel[6].Tjna;
      FC.x = (Pos[6].Tjna) - (INITIAL_COND[6].x);

      if(xarmAdjustTime( IC, FC, MaxTime, SDI_F->limitsGoto.angular, &(AxisTimes.QK), &Jerk_Transition[6], &DirTransition_a[6], &DirTransition_b[6])!=0 || AdjustTimeIsOk == 0) {
	// There is an error while adjusting times thus we stop the motion
	AdjustTimeIsOk = 0;

	if (tpGetMonotonicTimes(AxisTimes_End.QK, &TM_End, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
 	}
	if (tpGetMonotonicTimes(AxisTimes_Start.QK, &TM_Start, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement(&TM_End, &TNE_End);
	tpGetNumberOfElement(&TM_Start, &TNE_Start);

	tmEnd   = TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna + TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb;
	tmStart = TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna + TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;	    
	DiffTime = MaxTimeM - tmEnd - tmStart;
	J = SDI_F->limitsGoto.angular.maxJerk;

	/* Add Stop and Start Motion */
	queueAddDetails(&(SDI_F->Seg[6]), 1, TNE_End.Tjpa, J, INITIAL_COND[6].a, INITIAL_COND[6].v,  INITIAL_COND[6].x, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 2, TNE_End.Taca, J, Acc_End[6].Tjpa, Vel_End[6].Tjpa, Pos_End[6].Tjpa, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 3, TNE_End.Tjna, J, Acc_End[6].Taca, Vel_End[6].Taca, Pos_End[6].Taca, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 4, TNE_End.Tvc,  J, Acc_End[6].Tjna, Vel_End[6].Tjna, Pos_End[6].Tjna, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 5, TNE_End.Tjnb, J, Acc_End[6].Tvc,  Vel_End[6].Tvc,  Pos_End[6].Tvc,  DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 6, TNE_End.Tacb, J, Acc_End[6].Tjnb, Vel_End[6].Tjnb, Pos_End[6].Tjnb, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 7, TNE_End.Tjpb, J, Acc_End[6].Tacb, Vel_End[6].Tacb, Pos_End[6].Tacb, DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 4, DiffTime,  J, 0, 0, PosStop[6], DirTransition_End[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 1, TNE_Start.Tjpa, J, 0, 0, PosStop[6], DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 2, TNE_Start.Taca, J, Acc_Start[6].Tjpa, Vel_Start[6].Tjpa, Pos_Start[6].Tjpa,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 3, TNE_Start.Tjna, J, Acc_Start[6].Taca, Vel_Start[6].Taca, Pos_Start[6].Taca,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 4, TNE_Start.Tvc,  J, Acc_Start[6].Tjna, Vel_Start[6].Tjna, Pos_Start[6].Tjna,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 5, TNE_Start.Tjnb, J, Acc_Start[6].Tvc,  Vel_Start[6].Tvc,  Pos_Start[6].Tvc,   DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 6, TNE_Start.Tacb, J, Acc_Start[6].Tjnb, Vel_Start[6].Tjnb, Pos_Start[6].Tjnb,  DirTransition_Start[6]);
	queueAddDetails(&(SDI_F->Seg[6]), 7, TNE_Start.Tjpb, J, Acc_Start[6].Tacb, Vel_Start[6].Tacb, Pos_Start[6].Tacb,  DirTransition_Start[6]);		
	timeSQK = timeSQK + TNE_End.Tjpa + TNE_End.Taca + TNE_End.Tjna +  TNE_End.Tvc + TNE_End.Tjnb + TNE_End.Tacb + TNE_End.Tjpb + DiffTime + TNE_Start.Tjpa + TNE_Start.Taca + TNE_Start.Tjna +  TNE_Start.Tvc + TNE_Start.Tjnb + TNE_Start.Tacb + TNE_Start.Tjpb;
       
      } else {
	IC.x = INITIAL_COND[6].x ;
	FC.x = (Pos[6].Tjna) - (INITIAL_COND[6].x);
	Jerks.J1 = Jerk_Transition[6];
	if (xarmVerifyTimes_Dir_ab(DISTANCE_TOLERANCE_ANGULAR, FC.x, Jerks, IC, DirTransition_a[6], DirTransition_b[6], (AxisTimes.QK) , &FC, &Acc_T[6], &Vel_T[6], &Pos_T[6])!=OK) {
	  PRINT_ERROR(" Verify Times QK");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}      
	/* Get Number of Elements needed for Vectors Definition */
	if (tpGetMonotonicTimes((AxisTimes.QK), &TM, &NOE)!=OK) {
	  PRINT_ERROR(" Get MonotonicTimes ");
	  *report = S_xarm_ERROR_TP;
	  return ETHER;
	}
	tpGetNumberOfElement (&TM, &TNE_T[6]);
	TimeIsAdjusted[6] = 1;
      }
    }

    if (TimeIsAdjusted[0] == 1 && AdjustTimeIsOk == 1) {
	if (SDI_F->printData == 1) {
	  printf("PX bkp 7\n");
	}
      queueAddDetails(&(SDI_F->Seg[0]), 1, TNE_T[0].Tjpa, Jerk_Transition[0], INITIAL_COND[0].a, INITIAL_COND[0].v, INITIAL_COND[0].x, DirTransition_a[0]);
      queueAddDetails(&(SDI_F->Seg[0]), 2, TNE_T[0].Taca, Jerk_Transition[0], Acc_T[0].Tjpa, Vel_T[0].Tjpa, Pos_T[0].Tjpa,  DirTransition_a[0]);
      queueAddDetails(&(SDI_F->Seg[0]), 3, TNE_T[0].Tjna, Jerk_Transition[0], Acc_T[0].Taca, Vel_T[0].Taca, Pos_T[0].Taca,  DirTransition_a[0]);
      queueAddDetails(&(SDI_F->Seg[0]), 4, TNE_T[0].Tvc,  Jerk_Transition[0], Acc_T[0].Tjna, Vel_T[0].Tjna, Pos_T[0].Tjna,  DirTransition_a[0]);
      queueAddDetails(&(SDI_F->Seg[0]), 1, TNE_T[0].Tjnb, Jerk_Transition[0], Acc_T[0].Tvc,  Vel_T[0].Tvc,  Pos_T[0].Tvc,   DirTransition_b[0]);
      queueAddDetails(&(SDI_F->Seg[0]), 2, TNE_T[0].Tacb, Jerk_Transition[0], Acc_T[0].Tjnb, Vel_T[0].Tjnb, Pos_T[0].Tjnb,  DirTransition_b[0]);
      queueAddDetails(&(SDI_F->Seg[0]), 3, TNE_T[0].Tjpb, Jerk_Transition[0], Acc_T[0].Tacb, Vel_T[0].Tacb, Pos_T[0].Tacb,  DirTransition_b[0]);
      timeSX  = timeSX  + TNE_T[0].Tjpa + TNE_T[0].Taca + TNE_T[0].Tjna +  TNE_T[0].Tvc + TNE_T[0].Tjnb + TNE_T[0].Tacb + TNE_T[0].Tjpb;
    }
    if (TimeIsAdjusted[1] == 1 && AdjustTimeIsOk == 1) {
      queueAddDetails(&(SDI_F->Seg[1]), 1, TNE_T[1].Tjpa, Jerk_Transition[1], INITIAL_COND[1].a, INITIAL_COND[1].v, INITIAL_COND[1].x, DirTransition_a[1]);
      queueAddDetails(&(SDI_F->Seg[1]), 2, TNE_T[1].Taca, Jerk_Transition[1], Acc_T[1].Tjpa, Vel_T[1].Tjpa, Pos_T[1].Tjpa,  DirTransition_a[1]);
      queueAddDetails(&(SDI_F->Seg[1]), 3, TNE_T[1].Tjna, Jerk_Transition[1], Acc_T[1].Taca, Vel_T[1].Taca, Pos_T[1].Taca,  DirTransition_a[1]);
      queueAddDetails(&(SDI_F->Seg[1]), 4, TNE_T[1].Tvc,  Jerk_Transition[1], Acc_T[1].Tjna, Vel_T[1].Tjna, Pos_T[1].Tjna,  DirTransition_a[1]);
      queueAddDetails(&(SDI_F->Seg[1]), 1, TNE_T[1].Tjnb, Jerk_Transition[1], Acc_T[1].Tvc,  Vel_T[1].Tvc,  Pos_T[1].Tvc,   DirTransition_b[1]);
      queueAddDetails(&(SDI_F->Seg[1]), 2, TNE_T[1].Tacb, Jerk_Transition[1], Acc_T[1].Tjnb, Vel_T[1].Tjnb, Pos_T[1].Tjnb,  DirTransition_b[1]);
      queueAddDetails(&(SDI_F->Seg[1]), 3, TNE_T[1].Tjpb, Jerk_Transition[1], Acc_T[1].Tacb, Vel_T[1].Tacb, Pos_T[1].Tacb,  DirTransition_b[1]);
      timeSY  = timeSY  + TNE_T[1].Tjpa + TNE_T[1].Taca + TNE_T[1].Tjna +  TNE_T[1].Tvc + TNE_T[1].Tjnb + TNE_T[1].Tacb + TNE_T[1].Tjpb;
    } 
    if (TimeIsAdjusted[2] == 1 && AdjustTimeIsOk == 1) {
      queueAddDetails(&(SDI_F->Seg[2]), 1, TNE_T[2].Tjpa, Jerk_Transition[2], INITIAL_COND[2].a, INITIAL_COND[2].v, INITIAL_COND[2].x, DirTransition_a[2]);
      queueAddDetails(&(SDI_F->Seg[2]), 2, TNE_T[2].Taca, Jerk_Transition[2], Acc_T[2].Tjpa, Vel_T[2].Tjpa, Pos_T[2].Tjpa,  DirTransition_a[2]);
      queueAddDetails(&(SDI_F->Seg[2]), 3, TNE_T[2].Tjna, Jerk_Transition[2], Acc_T[2].Taca, Vel_T[2].Taca, Pos_T[2].Taca,  DirTransition_a[2]);
      queueAddDetails(&(SDI_F->Seg[2]), 4, TNE_T[2].Tvc,  Jerk_Transition[2], Acc_T[2].Tjna, Vel_T[2].Tjna, Pos_T[2].Tjna,  DirTransition_a[2]);
      queueAddDetails(&(SDI_F->Seg[2]), 1, TNE_T[2].Tjnb, Jerk_Transition[2], Acc_T[2].Tvc,  Vel_T[2].Tvc,  Pos_T[2].Tvc,   DirTransition_b[2]);
      queueAddDetails(&(SDI_F->Seg[2]), 2, TNE_T[2].Tacb, Jerk_Transition[2], Acc_T[2].Tjnb, Vel_T[2].Tjnb, Pos_T[2].Tjnb,  DirTransition_b[2]);
      queueAddDetails(&(SDI_F->Seg[2]), 3, TNE_T[2].Tjpb, Jerk_Transition[2], Acc_T[2].Tacb, Vel_T[2].Tacb, Pos_T[2].Tacb,  DirTransition_b[2]);
      timeSZ  = timeSZ  + TNE_T[2].Tjpa + TNE_T[2].Taca + TNE_T[2].Tjna +  TNE_T[2].Tvc + TNE_T[2].Tjnb + TNE_T[2].Tacb + TNE_T[2].Tjpb;
    }
    if (TimeIsAdjusted[3] == 1 && AdjustTimeIsOk == 1) {
      queueAddDetails(&(SDI_F->Seg[3]), 1, TNE_T[3].Tjpa, Jerk_Transition[3], INITIAL_COND[3].a, INITIAL_COND[3].v, INITIAL_COND[3].x, DirTransition_a[3]);
      queueAddDetails(&(SDI_F->Seg[3]), 2, TNE_T[3].Taca, Jerk_Transition[3], Acc_T[3].Tjpa, Vel_T[3].Tjpa, Pos_T[3].Tjpa,  DirTransition_a[3]);
      queueAddDetails(&(SDI_F->Seg[3]), 3, TNE_T[3].Tjna, Jerk_Transition[3], Acc_T[3].Taca, Vel_T[3].Taca, Pos_T[3].Taca,  DirTransition_a[3]);
      queueAddDetails(&(SDI_F->Seg[3]), 4, TNE_T[3].Tvc,  Jerk_Transition[3], Acc_T[3].Tjna, Vel_T[3].Tjna, Pos_T[3].Tjna,  DirTransition_a[3]);
      queueAddDetails(&(SDI_F->Seg[3]), 1, TNE_T[3].Tjnb, Jerk_Transition[3], Acc_T[3].Tvc,  Vel_T[3].Tvc,  Pos_T[3].Tvc,   DirTransition_b[3]);
      queueAddDetails(&(SDI_F->Seg[3]), 2, TNE_T[3].Tacb, Jerk_Transition[3], Acc_T[3].Tjnb, Vel_T[3].Tjnb, Pos_T[3].Tjnb,  DirTransition_b[3]);
      queueAddDetails(&(SDI_F->Seg[3]), 3, TNE_T[3].Tjpb, Jerk_Transition[3], Acc_T[3].Tacb, Vel_T[3].Tacb, Pos_T[3].Tacb,  DirTransition_b[3]);
      timeSQN = timeSQN + TNE_T[3].Tjpa + TNE_T[3].Taca + TNE_T[3].Tjna +  TNE_T[3].Tvc + TNE_T[3].Tjnb + TNE_T[3].Tacb + TNE_T[3].Tjpb;
    }
    if (TimeIsAdjusted[4] == 1 && AdjustTimeIsOk == 1) {
      queueAddDetails(&(SDI_F->Seg[4]), 1, TNE_T[4].Tjpa, Jerk_Transition[4], INITIAL_COND[4].a, INITIAL_COND[4].v, INITIAL_COND[4].x, DirTransition_a[4]);
      queueAddDetails(&(SDI_F->Seg[4]), 2, TNE_T[4].Taca, Jerk_Transition[4], Acc_T[4].Tjpa, Vel_T[4].Tjpa, Pos_T[4].Tjpa,  DirTransition_a[4]);
      queueAddDetails(&(SDI_F->Seg[4]), 3, TNE_T[4].Tjna, Jerk_Transition[4], Acc_T[4].Taca, Vel_T[4].Taca, Pos_T[4].Taca,  DirTransition_a[4]);
      queueAddDetails(&(SDI_F->Seg[4]), 4, TNE_T[4].Tvc,  Jerk_Transition[4], Acc_T[4].Tjna, Vel_T[4].Tjna, Pos_T[4].Tjna,  DirTransition_a[4]);
      queueAddDetails(&(SDI_F->Seg[4]), 1, TNE_T[4].Tjnb, Jerk_Transition[4], Acc_T[4].Tvc,  Vel_T[4].Tvc,  Pos_T[4].Tvc,   DirTransition_b[4]);
      queueAddDetails(&(SDI_F->Seg[4]), 2, TNE_T[4].Tacb, Jerk_Transition[4], Acc_T[4].Tjnb, Vel_T[4].Tjnb, Pos_T[4].Tjnb,  DirTransition_b[4]);
      queueAddDetails(&(SDI_F->Seg[4]), 3, TNE_T[4].Tjpb, Jerk_Transition[4], Acc_T[4].Tacb, Vel_T[4].Tacb, Pos_T[4].Tacb,  DirTransition_b[4]);
      timeSQI = timeSQI + TNE_T[4].Tjpa + TNE_T[4].Taca + TNE_T[4].Tjna +  TNE_T[4].Tvc + TNE_T[4].Tjnb + TNE_T[4].Tacb + TNE_T[4].Tjpb;
    }
    if (TimeIsAdjusted[5] == 1 && AdjustTimeIsOk == 1) {
      queueAddDetails(&(SDI_F->Seg[5]), 1, TNE_T[5].Tjpa, Jerk_Transition[5], INITIAL_COND[5].a, INITIAL_COND[5].v, INITIAL_COND[5].x, DirTransition_a[5]);
      queueAddDetails(&(SDI_F->Seg[5]), 2, TNE_T[5].Taca, Jerk_Transition[5], Acc_T[5].Tjpa, Vel_T[5].Tjpa, Pos_T[5].Tjpa,  DirTransition_a[5]);
      queueAddDetails(&(SDI_F->Seg[5]), 3, TNE_T[5].Tjna, Jerk_Transition[5], Acc_T[5].Taca, Vel_T[5].Taca, Pos_T[5].Taca,  DirTransition_a[5]);
      queueAddDetails(&(SDI_F->Seg[5]), 4, TNE_T[5].Tvc,  Jerk_Transition[5], Acc_T[5].Tjna, Vel_T[5].Tjna, Pos_T[5].Tjna,  DirTransition_a[5]);
      queueAddDetails(&(SDI_F->Seg[5]), 1, TNE_T[5].Tjnb, Jerk_Transition[5], Acc_T[5].Tvc,  Vel_T[5].Tvc,  Pos_T[5].Tvc,   DirTransition_b[5]);
      queueAddDetails(&(SDI_F->Seg[5]), 2, TNE_T[5].Tacb, Jerk_Transition[5], Acc_T[5].Tjnb, Vel_T[5].Tjnb, Pos_T[5].Tjnb,  DirTransition_b[5]);
      queueAddDetails(&(SDI_F->Seg[5]), 3, TNE_T[5].Tjpb, Jerk_Transition[5], Acc_T[5].Tacb, Vel_T[5].Tacb, Pos_T[5].Tacb,  DirTransition_b[5]);
      timeSQJ = timeSQJ + TNE_T[5].Tjpa + TNE_T[5].Taca + TNE_T[5].Tjna +  TNE_T[5].Tvc + TNE_T[5].Tjnb + TNE_T[5].Tacb + TNE_T[5].Tjpb;
    }   
    if (TimeIsAdjusted[6] == 1 && AdjustTimeIsOk == 1) {
      queueAddDetails(&(SDI_F->Seg[6]), 1, TNE_T[6].Tjpa, Jerk_Transition[6], INITIAL_COND[6].a, INITIAL_COND[6].v, INITIAL_COND[6].x, DirTransition_a[6]);
      queueAddDetails(&(SDI_F->Seg[6]), 2, TNE_T[6].Taca, Jerk_Transition[6], Acc_T[6].Tjpa, Vel_T[6].Tjpa, Pos_T[6].Tjpa,  DirTransition_a[6]);
      queueAddDetails(&(SDI_F->Seg[6]), 3, TNE_T[6].Tjna, Jerk_Transition[6], Acc_T[6].Taca, Vel_T[6].Taca, Pos_T[6].Taca,  DirTransition_a[6]);
      queueAddDetails(&(SDI_F->Seg[6]), 4, TNE_T[6].Tvc,  Jerk_Transition[6], Acc_T[6].Tjna, Vel_T[6].Tjna, Pos_T[6].Tjna,  DirTransition_a[6]);
      queueAddDetails(&(SDI_F->Seg[6]), 1, TNE_T[6].Tjnb, Jerk_Transition[6], Acc_T[6].Tvc,  Vel_T[6].Tvc,  Pos_T[6].Tvc,   DirTransition_b[6]);
      queueAddDetails(&(SDI_F->Seg[6]), 2, TNE_T[6].Tacb, Jerk_Transition[6], Acc_T[6].Tjnb, Vel_T[6].Tjnb, Pos_T[6].Tjnb,  DirTransition_b[6]);
      queueAddDetails(&(SDI_F->Seg[6]), 3, TNE_T[6].Tjpb, Jerk_Transition[6], Acc_T[6].Tacb, Vel_T[6].Tacb, Pos_T[6].Tacb,  DirTransition_b[6]);
      timeSQK = timeSQK + TNE_T[6].Tjpa + TNE_T[6].Taca + TNE_T[6].Tjna +  TNE_T[6].Tvc + TNE_T[6].Tjnb + TNE_T[6].Tacb + TNE_T[6].Tjpb;
    }

    //  printf("Before add segment \n");
    //  printf("timeSX %d \n",timeSX);
    //  printf("timeSY %d \n",timeSY);
    //  printf("timeSQN %d \n",timeSQN);
    //  printf("timeSQI %d \n",timeSQI);
    //  printf("timeSQJ %d \n",timeSQJ);
    //  printf("timeSQK %d \n",timeSQK);

    /* Add Segments Tvc of the second motion*/
    queueAddDetails(&(SDI_F->Seg[0]), 4, TNE.Tvc,  jerk[0].J1, Acc[0].Tjna, Vel[0].Tjna, Pos[0].Tjna, Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[1]), 4, TNE.Tvc,  jerk[1].J1, Acc[1].Tjna, Vel[1].Tjna, Pos[1].Tjna, Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[2]), 4, TNE.Tvc,  jerk[2].J1, Acc[2].Tjna, Vel[2].Tjna, Pos[2].Tjna, Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[3]), 4, TNE.Tvc,  jerk[3].J1, Acc[3].Tjna, Vel[3].Tjna, Pos[3].Tjna, Dir.el[0][3]);
    queueAddDetails(&(SDI_F->Seg[4]), 4, TNE.Tvc,  jerk[4].J1, Acc[4].Tjna, Vel[4].Tjna, Pos[4].Tjna, Dir.el[0][4]);
    queueAddDetails(&(SDI_F->Seg[5]), 4, TNE.Tvc,  jerk[5].J1, Acc[5].Tjna, Vel[5].Tjna, Pos[5].Tjna, Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[6]), 4, TNE.Tvc,  jerk[6].J1, Acc[6].Tjna, Vel[6].Tjna, Pos[6].Tjna, Dir.el[0][6]);

    //  timeSX  = timeSX + TNE.Tvc;
    //  timeSY  = timeSY  + TNE.Tvc;
    //  timeSZ  = timeSZ  + TNE.Tvc;
    //  timeSQN = timeSQN + TNE.Tvc;
    //  timeSQI = timeSQI + TNE.Tvc;
    //  timeSQJ = timeSQJ + TNE.Tvc;
    //  timeSQK = timeSQK + TNE.Tvc;
    //  printf("After add segment \n");
    //  printf("timeSX %d \n",timeSX);
    //  printf("timeSY %d \n",timeSY);
    //  printf("timeSQN %d \n",timeSQN);
    //  printf("timeSQI %d \n",timeSQI);
    //  printf("timeSQJ %d \n",timeSQJ);
    //  printf("timeSQK %d \n",timeSQK);

    for(i=0;i<=XARM_NB_DIM-1;i++) {
      INITIAL_COND[i].a =  Acc[i].Tvc;
      INITIAL_COND[i].v =  Vel[i].Tvc;  
      INITIAL_COND[i].x =  Pos[i].Tvc; 
    } 
    indexTrajectory ++; 

    /* Motion of arm start */  
    SDI_F->xarmTrackSegment = XARM_TRACK_ON;  
    return EXEC;
  }
  else {
    queueAddDetails(&(SDI_F->Seg[0]), 5, TNE.Tjnb, jerk[0].J1, Acc[0].Tvc,  Vel[0].Tvc,  Pos[0].Tvc, Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[0]), 6, TNE.Tacb, jerk[0].J1, Acc[0].Tjnb, Vel[0].Tjnb, Pos[0].Tjnb, Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[0]), 7, TNE.Tjpb, jerk[0].J1, Acc[0].Tacb, Vel[0].Tacb, Pos[0].Tacb, Dir.el[0][0]);
    queueAddDetails(&(SDI_F->Seg[1]), 5, TNE.Tjnb, jerk[1].J1, Acc[1].Tvc,  Vel[1].Tvc,  Pos[1].Tvc, Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[1]), 6, TNE.Tacb, jerk[1].J1, Acc[1].Tjnb, Vel[1].Tjnb, Pos[1].Tjnb, Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[1]), 7, TNE.Tjpb, jerk[1].J1, Acc[1].Tacb, Vel[1].Tacb, Pos[1].Tacb, Dir.el[0][1]);
    queueAddDetails(&(SDI_F->Seg[2]), 5, TNE.Tjnb, jerk[2].J1, Acc[2].Tvc,  Vel[2].Tvc,  Pos[2].Tvc, Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[2]), 6, TNE.Tacb, jerk[2].J1, Acc[2].Tjnb, Vel[2].Tjnb, Pos[2].Tjnb, Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[2]), 7, TNE.Tjpb, jerk[2].J1, Acc[2].Tacb, Vel[2].Tacb, Pos[2].Tacb, Dir.el[0][2]);
    queueAddDetails(&(SDI_F->Seg[3]), 5, TNE.Tjnb, jerk[3].J1, Acc[3].Tvc,  Vel[3].Tvc,  Pos[3].Tvc, Dir.el[0][3]);
    queueAddDetails(&(SDI_F->Seg[3]), 6, TNE.Tacb, jerk[3].J1, Acc[3].Tjnb, Vel[3].Tjnb, Pos[3].Tjnb, Dir.el[0][3]);
    queueAddDetails(&(SDI_F->Seg[3]), 7, TNE.Tjpb, jerk[3].J1, Acc[3].Tacb, Vel[3].Tacb, Pos[3].Tacb, Dir.el[0][3]);    
    queueAddDetails(&(SDI_F->Seg[4]), 5, TNE.Tjnb, jerk[4].J1, Acc[4].Tvc,  Vel[4].Tvc,  Pos[4].Tvc, Dir.el[0][4]);
    queueAddDetails(&(SDI_F->Seg[4]), 6, TNE.Tacb, jerk[4].J1, Acc[4].Tjnb, Vel[4].Tjnb, Pos[4].Tjnb, Dir.el[0][4]);
    queueAddDetails(&(SDI_F->Seg[4]), 7, TNE.Tjpb, jerk[4].J1, Acc[4].Tacb, Vel[4].Tacb, Pos[4].Tacb, Dir.el[0][4]);    
    queueAddDetails(&(SDI_F->Seg[5]), 5, TNE.Tjnb, jerk[5].J1, Acc[5].Tvc,  Vel[5].Tvc,  Pos[5].Tvc, Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[5]), 6, TNE.Tacb, jerk[5].J1, Acc[5].Tjnb, Vel[5].Tjnb, Pos[5].Tjnb, Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[5]), 7, TNE.Tjpb, jerk[5].J1, Acc[5].Tacb, Vel[5].Tacb, Pos[5].Tacb, Dir.el[0][5]);
    queueAddDetails(&(SDI_F->Seg[6]), 5, TNE.Tjnb, jerk[6].J1, Acc[6].Tvc,  Vel[6].Tvc,  Pos[6].Tvc, Dir.el[0][6]);
    queueAddDetails(&(SDI_F->Seg[6]), 6, TNE.Tacb, jerk[6].J1, Acc[6].Tjnb, Vel[6].Tjnb, Pos[6].Tjnb, Dir.el[0][6]);
    queueAddDetails(&(SDI_F->Seg[6]), 7, TNE.Tjpb, jerk[6].J1, Acc[6].Tacb, Vel[6].Tacb, Pos[6].Tacb, Dir.el[0][6]);

    /* Motion of arm start */
    // SDI_F->xarmTrackSegment = XARM_TRACK_ON;  

    return END;
  }
}

/* xarmPlanningTrajectoryEnd  -  codel END of PlanningTrajectory
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmPlanningTrajectoryEnd(int *report)
{
  if(SDI_F->saveFiles==1) {
    fclose(fileptr);
  }
  freemat(PFrom);
  freemat(PTo);
  freemat(QFrom);
  freemat(QTo);
  freemat(Dist);
  freemat(AbsDist);
  freemat(Dir);
  freemat(LDist);
  freemat(AbsLDist);
  freemat(ADist);
  freemat(AbsADist);
  printf("XARM PLANNING TRAJECTORY ENDED\n");
  return ETHER;
}

/* xarmPlanningTrajectoryInter  -  codel INTER of PlanningTrajectory
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
xarmPlanningTrajectoryInter(int *report)
{
  /* ... add your code here ... */
  return ETHER;
}




